// testing DGNorm2d

#include <iostream>

#include "toolbox.hh"
#include "basics/testsuite.hh"
#include "basics/exceptions.hh"
#include "geometry/meshImport2D.hh"
#include "function.hh"
#include "vectorial.hh"
#include "linDG2D.hh"

#include "dgNorm2d.hh"

#include "normTest.hh"

namespace test {
  
  // ******************************************************** DGNorm2dTest **

  void DGNorm2dTest::run() {
    concepts::Import2dMesh mesh (SOURCEDIR "/app-cwinkelm/0101.coord",
				 SOURCEDIR "/app-cwinkelm/0101.elms",
				 SOURCEDIR "/app-cwinkelm/0101.bnd", 1);
    linDG2D::SpaceP1 spc(mesh);
    concepts::Vector<Real> zeroVec(spc);

    vectorial::Space<Real> vSpc(2);
    vSpc.put(spc);
    vSpc.put(spc);
    concepts::Vector<Real> vZeroVec(vSpc);

    testNorms(zeroVec, vZeroVec);
  }

  void DGNorm2dTest::testNorms(const concepts::Vector<Real>& z,
                               const concepts::Vector<Real>& zv) {
    concepts::ParsedFormula zero("(0)");
    concepts::ParsedFormula one("(1)");
    concepts::ParsedFormula x("(x)");
    concepts::ParsedFormula y("(y)");
    concepts::ParsedFormula xy("(x+y)");

    _numtest(DGNorm2d::l2(z, zero, 2), 0);
    _numtest(DGNorm2d::l2(z, one, 2), 1);
    _numtest(DGNorm2d::l2(z, x, 2), std::sqrt(1./3.));
    _numtest(DGNorm2d::l2(z, xy, 2), std::sqrt(7./6.));

    _numtest(DGNorm2d::l2(zv, one, one, 2), std::sqrt(2.));
    _numtest(DGNorm2d::l2(zv, one, x, 2), std::sqrt(4./3.));
    _numtest(DGNorm2d::l2(zv, x, y, 2), std::sqrt(2./3.));

  }

}

int main(int, char**) {
  //  std::cout << std::setprecision(20);

  test::DGNorm2dTest t;
  t.run();

  uint nFailed = t.getNumFailed();
  if (nFailed) {
    t.report();
  } else {
    std::cout << "DGNorm2dTest: all tests passed" << std::endl;
  }
  return nFailed;
}
