#include "periodicSquare.hh"

PeriodicSquare::PeriodicSquare(uint n, Real jitter) : n_(n), jitter_(jitter) {
  if ( (jitter_ < -0.25) || (jitter > 0.25) ) {
    jitter_ = 0.25;
  }
  jitter_ /= static_cast<Real>(n_);
  // arrays
  coord_ = new concepts::Real2d*[(n_+1)*(n_+1)];
  vtx_ = new concepts::Vertex*[n_*n_];
  edg_ = new concepts::Edge*[3*n_*n_];
  tri_ = new concepts::Triangle*[2*n_*n_];
  cell_ = new concepts::Triangle2d*[2*n_*n_];
  // coordinates
  for (uint a=0; a<=n_; ++a) {
    for (uint b=0; b<=n_; ++b) {
      coord_[ii(a,b)] = new concepts::Real2d(x(a), x(b));
      if( (jitter_!=0) && (a%n_!=0) && (b%n_!=0) ) {
        *coord_[ii(a,b)] += concepts::Real2d(dRand(), dRand());
      }
    }
  }
  // vertices
  for (uint i = 0; i < n_*n_; ++i)
    vtx_[i] = new concepts::Vertex();
  // edges
  for(uint a=0; a<n_; ++a) {
    for(uint b=0; b<n_; ++b) {
      edg_[k(a,b,0)] = 
        new concepts::Edge(*vtx_[j(a,b)], *vtx_[j(a+1,b)]);
      edg_[k(a,b,1)] = 
        new concepts::Edge(*vtx_[j(a,b)], *vtx_[j(a+1,b+1)]);
      edg_[k(a,b,2)] = 
        new concepts::Edge(*vtx_[j(a,b)], *vtx_[j(a,b+1)]);
    }
  }

  // triangles
  for(uint a=0; a<n_; ++a) {
    for(uint b=0; b<n_; ++b) {
      tri_[l(a,b,0)] = new concepts::Triangle(*edg_[k(a,b,0)],
                                              *edg_[k(a+1,b,2)],
                                              *edg_[k(a,b,1)]);
      tri_[l(a,b,1)] = new concepts::Triangle(*edg_[k(a,b,1)],
                                              *edg_[k(a,b+1,0)],
                                              *edg_[k(a,b,2)]);
    }
  }

  // cells
  for(uint a=0; a<n_; ++a) {
    for(uint b=0; b<n_; ++b) {
      cell_[l(a,b,0)] = new concepts::Triangle2d(*tri_[l(a,b,0)],
	concepts::VertexTriangle2d(*(coord_[ii(a, b)]),
                                   *(coord_[ii(a+1, b)]),
                                   *(coord_[ii(a+1, b+1)])));
      cell_[l(a,b,1)] = new concepts::Triangle2d(*tri_[l(a,b,1)],
	concepts::VertexTriangle2d(*(coord_[ii(a, b)]),
                                   *(coord_[ii(a+1, b+1)]),
                                   *(coord_[ii(a, b+1)])));
    }
  }
}

PeriodicSquare::~PeriodicSquare() {
  for (uint i = 0; i < 2*n_*n_; ++i)
    delete cell_[i];
  delete [] cell_;
  for (uint i = 0; i < 2*n_*n_; ++i)
    delete tri_[i];
  delete [] tri_;
  for (uint i = 0; i < 3*n_*n_; ++i)
    delete edg_[i];
  delete [] edg_;
  for (uint i = 0; i < n_*n_; ++i)
    delete vtx_[i];
  delete [] vtx_;
  for (uint i = 0; i < (n_+1)*(n_+1); ++i)
    delete coord_[i];
  delete [] coord_;
}

std::ostream& PeriodicSquare::info(std::ostream& os) const {
  os << "PeriodicSquare(n = " << n_ << ")";
  return os;
}
