#include "square.hh"

// ****************************************************************** Square **

Square::Square(const Real sizex, const Real sizey) {

  for (uint i = 0; i < 4; ++i)
    vtx_[i] = new concepts::Vertex;

  edg_[0] = new concepts::Edge(*vtx_[0], *vtx_[1],1);
  edg_[1] = new concepts::Edge(*vtx_[1], *vtx_[2],2);
  edg_[2] = new concepts::Edge(*vtx_[2], *vtx_[3],3);
  edg_[3] = new concepts::Edge(*vtx_[0], *vtx_[3],4);

  quad_[0] = new concepts::Quad(*edg_[0], *edg_[1], *edg_[2], *edg_[3]);

  cell_[0] = new concepts::Quad2d
    (*quad_[0], 
     concepts::VertexQuad2d(concepts::Real3d(0.0, 0.0, 0.0),
			    concepts::Real3d(sizex, 0.0, 0.0),
			    concepts::Real3d(sizex, sizey, 0.0),
			    concepts::Real3d(0.0, sizey, 0.0))
     //concepts::MapQuad2d("(x,y)", 1.0, 1.0)
     );
}

Square::Square(const Real x0, const Real y0, const Real x1, const Real y1, 
	       const Real x2, const Real y2, const Real x3, const Real y3) {
for (uint i = 0; i < 4; ++i)
    vtx_[i] = new concepts::Vertex;

  edg_[0] = new concepts::Edge(*vtx_[0], *vtx_[1]);
  edg_[1] = new concepts::Edge(*vtx_[1], *vtx_[2]);
  edg_[2] = new concepts::Edge(*vtx_[2], *vtx_[3]);
  edg_[3] = new concepts::Edge(*vtx_[0], *vtx_[3]);

  quad_[0] = new concepts::Quad(*edg_[0], *edg_[1], *edg_[2], *edg_[3]);

  cell_[0] = new concepts::Quad2d
    (*quad_[0], 
     concepts::VertexQuad2d(concepts::Real3d(x0, y0, 0.0),
			    concepts::Real3d(x1, y1, 0.0),
			    concepts::Real3d(x2, y2, 0.0),
			    concepts::Real3d(x3, y3, 0.0))
     );
}


Square::~Square() {

  delete cell_[0];
  delete quad_[0];

  for (uint i = 0; i < 4; ++i) delete edg_[i];

  for (uint i = 0; i < 4; ++i) delete vtx_[i];
}

std::ostream& Square::info(std::ostream& os) const {

  os << "Square(";
  os << "area: " << *quad_[0]
     << ", cell: " << *cell_[0] << ")";

  return os;
}

// ***************************************************************** Square2 **

Square2::Square2() {

  for (uint i = 0; i < 6; ++i)
    vtx_[i] = new concepts::Vertex;

  edg_[0] = new concepts::Edge(*vtx_[0], *vtx_[1]);
  edg_[1] = new concepts::Edge(*vtx_[1], *vtx_[2]);
  edg_[2] = new concepts::Edge(*vtx_[2], *vtx_[3]);
  edg_[3] = new concepts::Edge(*vtx_[0], *vtx_[3]);
  edg_[4] = new concepts::Edge(*vtx_[2], *vtx_[4]);
  edg_[5] = new concepts::Edge(*vtx_[4], *vtx_[5]);
  edg_[6] = new concepts::Edge(*vtx_[5], *vtx_[3]);

  quad_[0] = new concepts::Quad(*edg_[0], *edg_[1], *edg_[2], *edg_[3]);
  quad_[1] = new concepts::Quad(*edg_[2], *edg_[4], *edg_[5], *edg_[6]);

  cell_[0] = new concepts::Quad2d
    (*quad_[0], concepts::MapQuad2d("(x,y/2)", 1.0, 1.0));
  cell_[1] = new concepts::Quad2d
    (*quad_[1], concepts::MapQuad2d("(x,1/2 + y/2)", 1.0, 1.0));
}

Square2::~Square2() {

  delete cell_[0];  delete cell_[1];
  delete quad_[0];  delete quad_[1];

  for (uint i = 0; i < 7; ++i) delete edg_[i];

  for (uint i = 0; i < 6; ++i) delete vtx_[i];
}

std::ostream& Square2::info(std::ostream& os) const {

  os << "Square2(";
  os << "areas: " << *quad_[0] << ", " << *quad_[1]
     << ", cells: " << *cell_[0] << ", " << *cell_[1] << ")";

  return os;
}
