/* FuCo
 * function compiler
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <memory.h>

#include "functionCompiler.hh"

#define FuCo_ParseStack      400
#define FuCo_ProcessStack    50
#define FuCo_Terminals       10
#define FuCo_NonTerminals    12

typedef struct {
  unsigned char id;
  union {
    double  a0;
    char    a1;
  } attrib;
} FuCo_token;

typedef unsigned short  FuCo_stack;
typedef unsigned long   FuCo_production;
 
static FuCo_production  productions[] = {
       0x00008382, 0x00048382, 0x00008584, 0x00058584, 0x00008786, 0x00068786,
       0x00000001, 0x00000002, 0x00000387, 0x00000487, 0x00078108, 0x00000000,
                   0x00000088, 0x00000a81, 0x000a818c, 0x0000818b, 0x00078a08 };
  
static unsigned char  parsingTbl[] = {
                   0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0xff, 0xff, 0xff,
                   0xff, 0xff, 0xff, 0x01, 0xff, 0xff, 0xff, 0x0b, 0x0b, 0x0b,
                   0x02, 0x02, 0x02, 0x02, 0xff, 0xff, 0x02, 0xff, 0xff, 0xff,
                   0xff, 0xff, 0xff, 0x0b, 0x03, 0xff, 0xff, 0x0b, 0x0b, 0x0b,
                   0x04, 0x04, 0x04, 0x04, 0xff, 0xff, 0x04, 0xff, 0xff, 0xff, 
                   0xff, 0xff, 0xff, 0x0b, 0x0b, 0x05, 0xff, 0x0b, 0x0b, 0x0b,
                   0x0c, 0x0c, 0x08, 0x09, 0xff, 0xff, 0x0c, 0xff, 0xff, 0xff,
                   0x06, 0x07, 0xff, 0xff, 0xff, 0xff, 0x0a, 0xff, 0xff, 0xff,
                   0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x10, 0xff, 0xff, 0xff,
                   0x0f, 0x0f, 0x0f, 0x0f, 0xff, 0xff, 0x0f, 0x0b, 0xff, 0xff,
                   0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x0b, 0xff, 0x0e,
                   0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x0b, 0xff, 0x0d };

/******************************************************************** lex() **/

static int lex(const char *s, FuCo_token *t)

{
  static char const  *c0 = NULL;
  char const         *c1 = NULL;
  char               c2[80];
  unsigned char state = 0;

  if (s != NULL) c0 = s;
  if (c0 == NULL) return(0);

  while (1) {

    switch (state) {
     case 0:
      while (*c0 == ' ') c0++;
      if ((*c0 == '*') || (*c0 == '/')) {
	t->id = 0x05; t->attrib.a1 = (*c0 == '*') ? 0 : 1;
	c0++; return(1);
      } else if ((*c0 == '+') || (*c0 == '-')) {
	t->id = 0x04; t->attrib.a1 = (*c0 == '+') ? 0 : 1;
	c0++; return(1);
      } else if (*c0 == '^') {
	t->id = 0x06; c0++; return(1);
      } else if (*c0 == '(') {
	t->id = 0x07; c0++; return(1);
      } else if (*c0 == ')') {
	t->id = 0x08; c0++; return(1);
      } else if (*c0 == '\0') {
	t->id = 0x09; c0 = NULL; return(1);
      } else if (*c0 == ',') {
	t->id = 0x0a; c0++; return(1);
      } else if (!memcmp(c0, "sinh", 4)) {
	t->id = 0x03; t->attrib.a1 = 7;
	c0 += 4; return(1);
      } else if (!memcmp(c0, "cosh", 4)) {
	t->id = 0x03; t->attrib.a1 = 8;
	c0 += 4; return(1);
      } else if (!memcmp(c0, "sin", 3)) {
	t->id = 0x03; t->attrib.a1 = 0;
	c0 += 3; return(1);
      } else if (!memcmp(c0, "cos", 3)) {
	t->id = 0x03; t->attrib.a1 = 1;
	c0 += 3; return(1);
      } else if (!memcmp(c0, "exp", 3)) {
	t->id = 0x03; t->attrib.a1 = 2;
	c0 += 3; return(1);
      } else if (!memcmp(c0, "ln", 2)) {
	t->id = 0x03; t->attrib.a1 = 3;
	c0 += 2; return(1);
      } else if (!memcmp(c0, "sqrt", 4)) {
	t->id = 0x03; t->attrib.a1 = 4;
	c0 += 4; return(1);
      } else if (!memcmp(c0, "sqr", 3)) {
	t->id = 0x03; t->attrib.a1 = 5;
	c0 += 3; return(1);
      } else if (!memcmp(c0, "atan", 4)) {
	t->id = 0x03; t->attrib.a1 = 6;
	c0 += 4; return(1);
      } else if (!memcmp(c0, "pi", 2)) {
	t->id = 0x02; t->attrib.a0 = M_PI;
	c0 += 2; return(1);
      } else if ((*c0 == 'x') || (*c0 == 'y') || (*c0 == 'z')) {
	t->id = 0x01; t->attrib.a1 = *c0 - 'x';
	c0++; return(1);
      } else if (isdigit(*c0)) {
	c1 = c0 + 1; state = 1;
      } else {
	c0 = NULL; return(0);
      }
      break;

     case 1:
      while (isdigit(*c1)) c1++;
      if (*c1 == '.') {
	c1++; state = 2;
      } else if (*c1 == 'E') {
	c1++; state = 3;
      } else {
	memcpy(c2, c0, c1 - c0); c2[c1 - c0] = 0;
	t->id = 0x02; t->attrib.a0 = atof(c2);
	c0 = c1; return(1);
      }
      break;

     case 2:
      if (isdigit(*c1)) {
	while (isdigit(*c1)) c1++;
	if (*c1 == 'E') {
	  c1++; state = 3;
	} else {
	  memcpy(c2, c0, c1 - c0); c2[c1 - c0] = 0;
	  t->id = 0x02; t->attrib.a0 = atof(c2);
	  c0 = c1; return(1);
	}
      } else {
	c0 = NULL; return(0);
      }
      break;

     case 3:
      if ((*c1 == '+') || (*c1 == '-'))
	c1++;
      if (isdigit(*c1)) {
	c1++;
	while (isdigit(*c1)) c1++;
	memcpy(c2, c0, c1 - c0); c2[c1 - c0] = 0;
	t->id = 0x02; t->attrib.a0 = atof(c2);
	c0 = c1; return(1);
      } else {
	c0 = NULL; return(0);
      }
      break;

    }
  }
}
	      
/****************************************************************** parse() **/

int parse(const char *sentence, unsigned char *pgm)

{
  FuCo_stack           stack[FuCo_ParseStack];
  register FuCo_stack* sp = stack + 1;
  FuCo_token           t;
  FuCo_production      p;
  unsigned char        pId; 
  unsigned char*       pgm0 = pgm;

  stack[0] = 0;

  *sp = 0x0089;
  if (!lex(sentence, &t)) return(0);

  while (*sp) {

    if (*sp & 0x0080) {
      pId = *(parsingTbl + ((*sp & 0x7f) - 1) * FuCo_Terminals + t.id - 1);
      if (pId == 0xff)
	return(0);
      else {

	switch (pId) {
	 case 0x01:
	  *sp = 0x0082; sp++; 
	  *sp = t.attrib.a1 ? 0x8002 : 0x8001; sp++;
	  *sp = 0x0083; sp++; *sp = 0x0004; sp++;
	  break;
	 case 0x03:
	  *sp = 0x0084; sp++; 
	  *sp = t.attrib.a1 ? 0x8004 : 0x8003; sp++;
	  *sp = 0x0085; sp++; *sp = 0x0005; sp++;
	  break;
	 case 0x05:
	  *sp = 0x0086; sp++; 
	  *sp = 0x8012; sp++; *sp = 0x8003; sp++;
	  *sp = 0x0087; sp++; 
	  *sp = 0x8013; sp++;
	  *sp = 0x0006; sp++;
	  break;
	 case 0x06:
	  *sp = t.attrib.a1 + 0x8021; sp++; 
	  *sp = 0x0001; sp++;
	  break;
	 case 0x07:
	  memcpy(sp, &t.attrib.a0, sizeof(double)); sp += sizeof(double);
	  *sp = 0x8020; sp++; 
	  *sp = 0x0002; sp++;
	  break;
	 case 0x08:
	  *sp = 0x8010 + t.attrib.a1; sp++; 
	  *sp = 0x0087; sp++; *sp = 0x0003; sp++;
	  break;
	 case 0x09:
	  if (t.attrib.a1) { *sp = 0x8005; sp++; }
	  *sp = 0x0087; sp++; *sp = 0x0004; sp++;
	  break;
	 case 0x0f:
	  *sp = 0x008b; sp++;
	  *sp = 0x80ff; sp++;
	  *sp = 0x0081; sp++;
	  break;
	 case 0x0e:
	  *sp = 0x008c; sp++;
	  *sp = 0x80ff; sp++;
	  *sp = 0x0081; sp++; *sp = 0x000a; sp++;
	  break;
	 case 0x0d:
	  *sp = 0x80ff; sp++;
	  *sp = 0x0081; sp++; *sp = 0x000a; sp++;
	  break;
	 default:
	  for (p = productions[pId]; p; p >>= 8)
	    *(sp++) = p & 0xff;
	}

      }
    } else {

      if (!((*sp == t.id) && lex(NULL, &t))) return(0);

    }

    sp--;

    while (*sp & 0x8000) {
      *pgm0 = *sp & 0xff; pgm0++;
      if (*sp == 0x8020) {
	sp -= sizeof(double);
	memcpy(pgm0, sp, sizeof(double));
	pgm0 += sizeof(double);
      }
      sp--;
    }
  }

  *pgm0 = 0x00;
  return(pgm0 - pgm + 1);
}

/**************************************************************** process() **/

void process(const unsigned char *pgm, double x, double y, double z, double v[])

{
  double           stack[FuCo_ProcessStack];
  register double  *sp = stack;
  register double  reg = 0;

  while (*pgm) {

    switch (*pgm) {
     case 0x01:
      reg += *sp; sp--; break;
     case 0x02:
      reg = *sp - reg; sp--; break;
     case 0x03:
      reg *= *sp; sp--; break;
     case 0x04:
      reg = *sp / reg; sp--; break;
     case 0x05:
      reg = -reg; break;
     case 0x10:
      reg = sin(reg); break;
     case 0x11:
      reg = cos(reg); break;
     case 0x12:
      reg = exp(reg); break;
     case 0x13:
      reg = log(reg); break;
     case 0x14:
      reg = sqrt(reg); break;
     case 0x15:
      reg *= reg; break;
     case 0x16:
      reg = atan(reg); break;
     case 0x17:
      reg = sinh(reg); break;
     case 0x18:
      reg = cosh(reg); break;
     case 0x20:
      memcpy(stack, pgm + 1, sizeof(double)); pgm += sizeof(double);
      sp++; *sp = reg; reg = *stack;
      break;
     case 0x21:
      sp++; *sp = reg; reg = x; break;
     case 0x22:
      sp++; *sp = reg; reg = y; break;
     case 0x23:
      sp++; *sp = reg; reg = z; break;
     case 0xff:
      *(v++) = reg; break;
    }

    pgm++;
  }
}
