// element for constraints

#ifndef constrElement_hh
#define constrElement_hh

#include "space/element.hh"
#include "space/elementPairs.hh"
#include "space/tmatrix.hh"

namespace constraints {

  // *************************************************************** Element **

  /** Element of the constraints space.
      @author Philipp Frauenfelder, 2002
   */
  template<typename F>
  class Element : public concepts::Element<F>,
		  public concepts::ElementPair<F> {
  public:
    /** Constructor.
	@param T List of T columns which describe the assembling of the local
	       into the global constraints matrix
    */
    Element(concepts::Element<F>& elm, concepts::TColumn<F>* T);
    virtual ~Element();
    virtual const concepts::TMatrix<F>& T() const { return T_; }

    /// Returns the computational element
    const concepts::Element<F>& compElement() const { return elm_; }

    virtual const concepts::Element<F>& elm1() const { return *this; }
    virtual const concepts::Element<F>& elm2() const { return elm_; }
  protected:
    virtual std::ostream& info(std::ostream& os) const;
  private:
    /// Element in the computational space
    concepts::Element<F>& elm_;

    /// TMatrix
    concepts::TMatrix<F> T_;    
  };

} // namespace constraints

#endif // constrElement_hh
