/* Basis for the functions
 */

#include "basis.hh"
#include "basics/typedefs.hh"

namespace concepts {

  // ************************************************************** Function **

  template<class F>
  Function<F>& Function<F>::operator+=(const Function<F>& fnc) {

    for (uint i = spc_.dim(); i--;) (*this)(i) += fnc(i);
    return *this;
  }

  template<class F>
  Function<F>& Function<F>::operator+=(F c) {

    for (uint i = spc_.dim(); i--;) (*this)(i) += c;
    return *this;
  }

  template<class F>
  Function<F>& Function<F>::operator-=(const Function<F>& fnc) {

    for (uint i = spc_.dim(); i--;) (*this)(i) -= fnc(i);
    return *this;
  }

  template<class F>
  Function<F>& Function<F>::operator-=(F c) {

    for (uint i = spc_.dim(); i--;) (*this)(i) -= c;
    return *this;
  }

  template<class F>
  Function<F>& Function<F>::operator*=(F a) {

    for (uint i = spc_.dim(); i--;) (*this)(i) *= a;
    return *this;
  }

  template<class F>
  Function<F>& Function<F>::add(const Function<F>& fnc, F a) {

    for (uint i = spc_.dim(); i--;) (*this)(i) += a * fnc(i);
    return *this;
  }

  template<class F>
  std::ostream& Function<F>::info(std::ostream& os) const {
    return os << "Function()";
  }

} // namespace concepts
