/* Basis for the functions
 */

#ifndef basis_hh
#define basis_hh

#include "basics/outputOperator.hh"
#include "space/space.hh"

namespace concepts {

  // ************************************************************** Function **

  /** Abstract class for a function. A function is always associated to
      a space which determines the dimension of the function.
  */
  template<class F>
  class Function : public OutputOperator {
  public:
    /// Constructor
    inline Function(const Space<F>& spc) : spc_(spc) {}

    /// Assignment operator
    virtual Function<F>& operator=(const Function<F>& fnc) = 0;

    /// Index operator
    virtual F& operator()(uint i) = 0;
    /// Index operator
    virtual F operator()(uint i) const = 0;

    /// Addition operator
    virtual Function<F>& operator+=(const Function<F>& fnc);
    /// Addition operator
    virtual Function<F>& operator+=(F c);

    /// Subtraction operator
    virtual Function<F>& operator-=(const Function<F>& fnc);
    /// Subtraction operator
    virtual Function<F>& operator-=(F c);

    /// Scaling operator
    virtual Function<F>& operator*=(F a);

    /// Adds \c a times \c fnc to this function
    virtual Function<F>& add(const Function<F>& fnc, F a);

    /// Returns the space on which the function is defined
    const Space<F>& space() const { return spc_; }
  protected:
    virtual std::ostream& info(std::ostream& os) const;
  private:
    /// Space on which the function is defined
    const Space<F>& spc_;
  };

} // namespace concepts

#endif // basis_hh
