/** Formula given element-wise, the data is given as a vector on a FE space
    @author Philipp Frauenfelder, 2004
*/

#include "function/elementFormula.hh"
#include "space/function.hh"
#include "space/tmatrix.hh"

namespace concepts {

  // ************************************************** ElementFormulaVector **

  template<typename F>
  Real ElementFormulaVector<F>::operator() (const Element<F>& elm,
					    const Real p, const Real t) const {
    Array<Real> res(1), coeff;
    elm.T().extract(v_, coeff);
    f_(elm, coeff, res, p, t);
    return res[0];
  }

  template<typename F>
  Real ElementFormulaVector<F>::operator() (const Element<F>& elm,
					    const Real2d& p,
					    const Real t) const {
    Array<Real> res(1), coeff;
    elm.T().extract(v_, coeff);
    f_(elm, coeff, res, p, t);
    return res[0];
  }

  template<typename F>
  Real ElementFormulaVector<F>::operator() (const Element<F>& elm,
					    const Real3d& p,
					    const Real t) const {
    Array<Real> res(1), coeff;
    elm.T().extract(v_, coeff);
    f_(elm, coeff, res, p, t);
    return res[0];
  }

  template class ElementFormulaVector<Real>;

} // namespace concepts
