/** Formula given element-wise, the data is given as a vector on a FE space
    @author Philipp Frauenfelder, 2004
*/

#ifndef elementFormVector_hh
#define elementFormVector_hh

#include "space/element.hh"
#include "function/vector.hh"

namespace concepts {

  template<typename F>
  class ScalarElementFunction;

  // ************************************************** ElementFormulaVector **

  /** Formula created from a FE function.
      The FE function is described by a vector of coefficients.
      @author Philipp Frauenfelder, 2004
  */
  template<typename F>
  class ElementFormulaVector : public ElementFormula<F> {
  public:
    /** Constructor
	@param v Vector of coefficients of the FE function
	@param f Function to be evaluated on the FE function, eg. hp2D::Value
    */
    ElementFormulaVector(const Vector<F>& v,
			 const ScalarElementFunction<F>& f) : v_(v), f_(f) {}
    virtual Real operator() (const Element<F>& elm, const Real p,
			     const Real t = 0.0) const;
    virtual Real operator() (const Element<F>& elm, const Real2d& p,
			     const Real t = 0.0) const;
    virtual Real operator() (const Element<F>& elm, const Real3d& p,
			     const Real t = 0.0) const;
    virtual ElementFormula<F>* clone() const {
      return new ElementFormulaVector(v_, f_); }
  private:
    const Vector<F>& v_;
    const ScalarElementFunction<F>& f_;
  };

} // namespace concepts

#endif // elementFormVector_hh
