/* Function derivation: Vector, stores the values in an array
 */

#include "vector.cc"

namespace concepts {

  // ************************************************************** Vector **

  template<>
  Vector<Cmplx>::Vector(const Space<Cmplx>& spc, const std::string& fname) : 
    Function<Cmplx>(spc), n_(spc.dim()), data_(n_), v_((Cmplx*)data_) {

    std::auto_ptr<std::ifstream> ifs(new std::ifstream(fname.c_str()));
    if (!(*ifs)) throw conceptsException(concepts::FileOpenError(fname));

    std::string line;
    std::getline(*ifs, line); // get one line from the file
    std::getline(*ifs, line); // get one line from the file
    uint n;
    std::istringstream iline(line.c_str());
    iline >> n;
    if (n_ != n) throw conceptsException(DimensionMismatch());
    std::getline(*ifs, line); // get one line from the file

    for (uint j = 0; j < n_; ++j) {
#ifdef __sparc__
      for (int i = sizeof(Cmplx)/2; i--;)
	ifs->read( ((char*)(v_+j)) + i, 1 );
      for (int i = sizeof(Cmplx)/2; i--;)
	ifs->read( ((char*)(v_+j)) + i + sizeof(Cmplx)/2, 1 );
#endif
#if defined(__i386__) || defined(__amd64__)
      ifs->read((char*)(v_+j), sizeof(Cmplx));
#endif
    }
  }

  template<>
  Real Vector<Cmplx>::l1() const {
    Real  l1 = 0.0;
    const Cmplx* v = v_;
    for (uint i = n_; i; --i) { l1 += std::abs(*v++); }
    return l1;
  }

  template<>
  Real Vector<Cmplx>::l2_2() const {
    Real  l2_2 = 0.0;
    Cmplx* v = v_;
    for (uint i = n_; i; --i) { l2_2 += std::norm(*v++); }
    return l2_2;
  }

  template<>
  Real Vector<Cmplx>::max() const {
    Real  max = 0.0;
    const Cmplx* v = v_;
    for (uint i = n_; i; --i) { 
      Real foo = std::abs(*v++);
      if (foo > max) max = foo;
    }
    return max;
  }

  template<>
  const Vector<Cmplx>& Vector<Cmplx>::write(const std::string& fname) const {
    std::ofstream ofs(fname.c_str());
    ofs << "# dimension of space" << std::endl
	<< n_ << std::endl;
    ofs << "# data in intel notation (take care of endianness)" << std::endl;
    for (uint j = 0; j < n_; ++j) {
#ifdef __sparc__
      for (int i = sizeof(Cmplx)/2; i--;)
	ofs.write( ((char*)(v_+j)) + i, 1 );
      for (int i = sizeof(Cmplx)/2; i--;)
	ofs.write( ((char*)(v_+j)) + i + sizeof(Cmplx)/2, 1 );
#endif
#if defined(__i386__) || defined(__amd64__)
      ofs.write((char*)(v_+j), sizeof(Cmplx));
#endif
    }
    return *this;
  }

  template class Vector<Cmplx>;

} // namespace concepts
