/* Function derivation: Vector, stores the values in an array
 */

#include "vector.cc"

namespace concepts {

  // ************************************************************** Vector **

  template<>
  Vector<Real>::Vector(const Space<Real>& spc, const std::string& fname) : 
    Function<Real>(spc), n_(spc.dim()), data_(n_), v_((Real*)data_) {

    std::auto_ptr<std::ifstream> ifs(new std::ifstream(fname.c_str()));
    if (!(*ifs)) throw conceptsException(concepts::FileOpenError(fname));

    std::string line;
    std::getline(*ifs, line); // get one line from the file
    std::getline(*ifs, line); // get one line from the file
    uint n;
    std::istringstream iline(line.c_str());
    iline >> n;
    if (n_ != n) throw conceptsException(DimensionMismatch());
    std::getline(*ifs, line); // get one line from the file

    for (uint j = 0; j < n_; ++j) {
#ifdef __sparc__
      for (int i = sizeof(Real); i--;)
	ifs->read( ((char*)(v_+j)) + i, 1 );
#endif
#if defined(__i386__) || defined (__amd64__)
      ifs->read((char*)(v_+j), sizeof(Real));
#endif
    }
  }

  template<>
  Real Vector<Real>::l1() const {

    Real  l1 = 0.0;
    const Real* v = v_;
    for (uint i = n_; i; --i) { l1 += *v < 0.0 ? -*v : *v; ++v; }

    return l1;
  }

  template<>
  Real Vector<Real>::l2_2() const {

    Real  l2_2 = 0.0;
    Real* v = v_;
    for (uint i = n_; i; --i) { l2_2 += *v * *v; ++v; }

    return l2_2;
  }

  template<>
  Real Vector<Real>::max() const {

    Real  max = 0.0;
    const Real* v = v_;
    for (uint i = n_; i; --i) { 
      Real foo = *v++; if (foo < 0.0) foo = -foo;
      if (foo > max) max = foo;
    }

    return max;
  }

  template<>
  const Vector<Real>& Vector<Real>::write(const std::string& fname) const {
    std::ofstream ofs(fname.c_str());
    ofs << "# dimension of space" << std::endl
	<< n_ << std::endl;
    ofs << "# data in intel notation (take care of endianness)" << std::endl;
    for (uint j = 0; j < n_; ++j) {
#ifdef __sparc__
      for (int i = sizeof(Real); i--;)
	ofs.write( ((char*)(v_+j)) + i, 1 );
#endif
#if defined(__i386__) || defined(__amd64__)
      ofs.write((char*)(v_+j), sizeof(Real));
#endif
    }
    return *this;
  }

  template class Vector<Real>;

} // namespace concepts
