// exception throw when illegal subdivision strategy changes occur

#ifndef stratChange_hh
#define stratChange_hh

#include "basics/exceptions.hh"
#include "subdivision.hh"

namespace concepts {

  // forward declaration
  class Connector;

  // ******************************************************** StrategyChange **

  /** Exception indicating that changing the subdivision strategy
      is not allowed (but was tried anyway).

      @author Philipp Frauenfelder, 2002
      @ingroup geometry
  */
  class StrategyChange : public ExceptionBase {
  public:
    /** Constructor.
	@param oldStrategy The current strategy
	@param newStrategy The requested strategy
	@param conn Connector on which the changed should have happened
    */
    StrategyChange(const Subdivision& oldStrategy,
		   const Subdivision& newStrategy,
		   const Connector& conn) throw();
    /// Returns the old strategy
    const Subdivision& oldStrategy() const throw() { return oldStrategy_; }
    /// Returns the new strategy
    const Subdivision& newStrategy() const throw() { return newStrategy_; }
    /// Returns the quad
    const Connector& connector() const throw() { return conn_; }
  protected:
    virtual std::ostream& info(std::ostream& os) const throw();
  private:
    const Subdivision& oldStrategy_, & newStrategy_;
    const Connector& conn_;
  };

} // namespace concepts

#endif // stratChange_hh
