/* testsuite for connector classes in geometry
 * shows how to write a testsuite for a small part of the library
 */

#include <iostream>
#include <memory>
#include "geometry/topology.hh"

using namespace concepts;

int main(int, char**) {

  std::cout << "Vertices:" << std::endl;
  Vertex v1(1);
  std::cout << "  " << v1 << std::endl;
  std::cout << "  child: " << *(v1.child(0)) << std::endl;
  std::auto_ptr<Vertex> vp(v1.clone());
  std::cout << "  clone: " << *vp << std::endl;

  std::cout << "Edges:" << std::endl;
  Vertex v2(2);
  Edge e1(v1, v2);
  std::cout << "  " << e1 << std::endl;
  std::auto_ptr<Edge> ep(e1.clone(*vp, v2));
  std::cout << "  clone: " << *ep << std::endl;

  std::cout << "Triangles:" << std::endl;
  Vertex v3(3);
  Edge e2(v2, v3);
  Edge e3(v3, v1);
  Triangle t1(e1, e2, e3);
  std::cout << "  " << t1 << std::endl;
  for (uint j = 0; j < 3; ++j)
    std::cout << "  child " << j << ": " << *(t1.child(j)) << std::endl;
  std::auto_ptr<Triangle> tp(t1.clone(*ep, e2, e3));
  std::cout << "  clone: " << *tp << std::endl;

  std::cout << "Quads:" << std::endl;
  Vertex v4;
  Edge e4(v3, v4);
  Edge e5(v4, v1);
  Quad q1(e1, e2, e4, e5);
  std::cout << "  " << q1 << std::endl;
  for (uint j = 0; j < 4; ++j)
    std::cout << "  child " << j << ": " << *(q1.child(j)) << std::endl;
  std::auto_ptr<Quad> qp(q1.clone(*ep, e2, e4, e5));
  std::cout << "  clone: " << *qp << std::endl;

  return 0;
}
