/* testing hexsubdiv
 */

#include <memory>
#include "geometry/mesh.hh"

// [0,1]^3 in two hexahedrons.
class Cube : public concepts::Mesh3 {
public:
  static const uint ncell_ = 1;

  Cube(const uint variant);
  virtual ~Cube();

  inline uint ncell() const { return ncell_; }
  inline concepts::Scan3* scan() { return new S(cell_); }

  virtual std::ostream& info(std::ostream& os) const;
private:
  class S : public concepts::Scan<concepts::Cell3> {
    uint idx_;
    concepts::Hexahedron3d *(&cell_)[ncell_];
  public:
    inline S(concepts::Hexahedron3d *(&cell)[ncell_]) : idx_(0), cell_(cell) {}
    inline S(const S &scan) : idx_(scan.idx_), cell_(scan.cell_) {}

    inline bool eos() const { return idx_ == ncell_; }
    inline concepts::Hexahedron3d& operator++(int) { return *cell_[idx_++]; }

    inline concepts::Scan3* clone() const { return new S(*this); }
  };
  concepts::Vertex* vtx_[8];
  concepts::Edge* edg_[12];
  concepts::Quad* quad_[6];
  concepts::Hexahedron* hex_[ncell_];

  concepts::Hexahedron3d* cell_[ncell_];
};

Cube::Cube(const uint variant) {
  for (uint i = 0; i < 8; ++i) vtx_[i] = new concepts::Vertex;

  edg_[0] = new concepts::Edge(*vtx_[0], *vtx_[1]);
  edg_[1] = new concepts::Edge(*vtx_[1], *vtx_[2]);
  edg_[2] = new concepts::Edge(*vtx_[2], *vtx_[3]);
  edg_[3] = new concepts::Edge(*vtx_[0], *vtx_[3]);
  edg_[4] = new concepts::Edge(*vtx_[0], *vtx_[4]);
  edg_[5] = new concepts::Edge(*vtx_[1], *vtx_[5]);
  edg_[6] = new concepts::Edge(*vtx_[2], *vtx_[6]);
  edg_[7] = new concepts::Edge(*vtx_[3], *vtx_[7]);
  edg_[8] = new concepts::Edge(*vtx_[4], *vtx_[5]);
  edg_[9] = new concepts::Edge(*vtx_[5], *vtx_[6]);
  edg_[10] = new concepts::Edge(*vtx_[6], *vtx_[7]);
  edg_[11] = new concepts::Edge(*vtx_[7], *vtx_[4]);

  switch(variant) {
  case 0:
    quad_[0] = new concepts::Quad(*edg_[0], *edg_[1], *edg_[2], *edg_[3], 1);
    quad_[1] = new concepts::Quad(*edg_[0], *edg_[5], *edg_[8], *edg_[4], 2);
    quad_[2] = new concepts::Quad(*edg_[1], *edg_[6], *edg_[9], *edg_[5], 3);
    quad_[3] = new concepts::Quad(*edg_[2], *edg_[6], *edg_[10], *edg_[7], 4);
    quad_[4] = new concepts::Quad(*edg_[3], *edg_[7], *edg_[11], *edg_[4], 5);
    quad_[5] = new concepts::Quad(*edg_[8], *edg_[9], *edg_[10], *edg_[11], 6);
    break;
  case 1:
    quad_[0] = new concepts::Quad(*edg_[1], *edg_[2], *edg_[3], *edg_[0], 1);
    quad_[1] = new concepts::Quad(*edg_[5], *edg_[8], *edg_[4], *edg_[0], 2);
    quad_[2] = new concepts::Quad(*edg_[6], *edg_[9], *edg_[5], *edg_[1], 3);
    quad_[3] = new concepts::Quad(*edg_[6], *edg_[10], *edg_[7], *edg_[2], 4);
    quad_[4] = new concepts::Quad(*edg_[7], *edg_[11], *edg_[4], *edg_[3], 5);
    quad_[5] = new concepts::Quad(*edg_[9], *edg_[10], *edg_[11], *edg_[8], 6);
    break;
  case 2:
    quad_[0] = new concepts::Quad(*edg_[2], *edg_[3], *edg_[0], *edg_[1], 1);
    quad_[1] = new concepts::Quad(*edg_[8], *edg_[4], *edg_[0], *edg_[5], 2);
    quad_[2] = new concepts::Quad(*edg_[9], *edg_[5], *edg_[1], *edg_[6], 3);
    quad_[3] = new concepts::Quad(*edg_[10], *edg_[7], *edg_[2], *edg_[6], 4);
    quad_[4] = new concepts::Quad(*edg_[11], *edg_[4], *edg_[3], *edg_[7], 5);
    quad_[5] = new concepts::Quad(*edg_[10], *edg_[11], *edg_[8], *edg_[9], 6);
    break;
  case 3:
    quad_[0] = new concepts::Quad(*edg_[3], *edg_[2], *edg_[1], *edg_[0], 1);
    quad_[1] = new concepts::Quad(*edg_[4], *edg_[8], *edg_[5], *edg_[0], 2);
    quad_[2] = new concepts::Quad(*edg_[5], *edg_[9], *edg_[6], *edg_[1], 3);
    quad_[3] = new concepts::Quad(*edg_[7], *edg_[10], *edg_[6], *edg_[2], 4);
    quad_[4] = new concepts::Quad(*edg_[4], *edg_[11], *edg_[7], *edg_[3], 5);
    quad_[5] = new concepts::Quad(*edg_[11], *edg_[10], *edg_[9], *edg_[8], 6);
    break;
  case 4:
    quad_[0] = new concepts::Quad(*edg_[2], *edg_[1], *edg_[0], *edg_[3], 1);
    quad_[1] = new concepts::Quad(*edg_[8], *edg_[5], *edg_[0], *edg_[4], 2);
    quad_[2] = new concepts::Quad(*edg_[9], *edg_[6], *edg_[1], *edg_[5], 3);
    quad_[3] = new concepts::Quad(*edg_[10], *edg_[6], *edg_[2], *edg_[7], 4);
    quad_[4] = new concepts::Quad(*edg_[11], *edg_[7], *edg_[3], *edg_[4], 5);
    quad_[5] = new concepts::Quad(*edg_[10], *edg_[9], *edg_[8], *edg_[11], 6);
    break;
  default: throw conceptsException(concepts::ExceptionBase());
  }

  hex_[0] = new concepts::Hexahedron(*quad_[0], *quad_[1], *quad_[2],
				     *quad_[3], *quad_[4], *quad_[5]);

  cell_[0] = new concepts::Hexahedron3d
    (*hex_[0], 
     concepts::MapHexahedron3d(concepts::Real3d(0.0, 0.0, 0.0),
			       concepts::Real3d(1.0, 0.0, 0.0),
			       concepts::Real3d(1.0, 1.0, 0.0),
			       concepts::Real3d(0.0, 1.0, 0.0),
			       concepts::Real3d(0.0, 0.0, 1.0),
			       concepts::Real3d(1.0, 0.0, 1.0),
			       concepts::Real3d(1.0, 1.0, 1.0),
			       concepts::Real3d(0.0, 1.0, 1.0))
     );
}

Cube::~Cube() {
  delete cell_[0]; delete hex_[0];
  for (uint i = 0; i < 6; ++i) delete quad_[i];
  for (uint i = 0; i < 12; ++i) delete edg_[i];
  for (uint i = 0; i < 8; ++i) delete vtx_[i];
}

std::ostream& Cube::info(std::ostream& os) const {
  return os << "Cube(volume: " << *hex_[0] << ", cell: " << *cell_[0] << ")";
}

int main(int, char**) {
  concepts::Stacktrace::doit() = false;
  try {
    const concepts::Hex3dSubdivision* strategy[7] = {
      concepts::Hex3dSubdiv8::instance(), concepts::Hex3dSubdiv2x::instance(),
      concepts::Hex3dSubdiv2y::instance(), concepts::Hex3dSubdiv2z::instance(),
      concepts::Hex3dSubdiv4x::instance(), concepts::Hex3dSubdiv4y::instance(),
      concepts::Hex3dSubdiv4z::instance() };
    uint variant = 0;
    do {
      std::cout << "--" << std::endl
		<< "variant " << variant << std::endl;
      for (uint subdiv = 0; subdiv < 7; ++subdiv) {
	Cube msh(variant);
	std::cout << "--" << std::endl
		  << "  " << *(strategy[subdiv]) << std::endl
		  << "  " << msh << std::endl;
	std::auto_ptr<concepts::Scan3> sc(msh.scan());
	while (*sc) {
	  concepts::Cell3& cell = (*sc)++;
	  std::cout << "  " << cell << std::endl;
	  concepts::Hexahedron3d* cellH =
	    dynamic_cast<concepts::Hexahedron3d*>(&cell);
	  cellH->setStrategy(strategy[subdiv]);
	  std::cout << "  " << *(cell.child(0)) << std::endl;
	}
      } // for subdiv
    } while (++variant);
  }
  catch(concepts::ExceptionBase& e) {
    std::cout << e << std::endl;
    std::cout << "First non-existing variant answers with an exception!"
	      << std::endl;
  }

  return 0;
}
