/* testsuite for 1D mesh classes in geometry
 */

#include <iostream>
#include "basics.hh"
#include "geometry.hh"

using namespace concepts;

class testMesh : public Mesh1 {
private:
  class S : public Scan<Cell1> {
    unsigned int idx_;
    Cell1 *(&cell_)[2];
  public:
    inline S(Cell1 *(&cell)[2]) : idx_(0), cell_(cell) {}
    inline S(const S& scan) : idx_(scan.idx_),  cell_(scan.cell_) {}
    inline bool eos() const { return idx_ == 2; }
    inline Cell1& operator++(int) { return *cell_[idx_++]; }
    inline Scan1* clone() const { return new S(*this); }
  };

  Vertex *vtx_[3];
  Edge *edg_[2];
  Cell1 *cell_[2];

public:
  testMesh();
  virtual ~testMesh();
  inline unsigned int ncell() const { return 2; }
  inline Scan1* scan() { return new S(cell_); }
  virtual std::ostream& info(std::ostream& os) const;
};

testMesh::testMesh() {

  for (int i = 0; i < 3; ++i)
    vtx_[i] = new Vertex(1);

  edg_[0] = new Edge(*vtx_[0], *vtx_[1], 1);
  edg_[1] = new Edge(*vtx_[1], *vtx_[2], 2);

  cell_[0] = new Edge1d(*edg_[0], MapEdge1d(1.0, 2.0));
  cell_[1] = new Edge1d(*edg_[1], MapEdge1d(2.0, 5.0));
}

testMesh::~testMesh() {
  for (int i = 0; i < 2; ++i)
    delete cell_[i];
  for (int i = 0; i < 2; ++i)
    delete edg_[i];
  for (int i = 0; i < 3; ++i)
    delete vtx_[i];
}

std::ostream& testMesh::info(std::ostream& os) const {
  os << "testMesh(" << *cell_[0] << ", " << *cell_[1] << ')';
  return os;
}

int main(int, char**) {

  testMesh t;
  std::cout << "Mesh: " << t << std::endl;

  return 0;
}
