/* 
TESTBLENDING
programme shows the functionality of the 
element maps using the Blending Function Method

Kersten Schmidt, 2004
 */

#include <iostream>
#include "basics.hh"
#include "geometry.hh"

using concepts::Real;
using concepts::Real2d;

/* <h3>Main program</h3>
*/
// ************************************************************ Main Program **

int main(int argc, char** argv) {

  try {
    uint n = 10;

    // four vertices
    Real2d vtx[4] = {Real2d(2.0,0.0), Real2d(0.0,2.0),
		     Real2d(-2.0,0.0), Real2d(0.0,-2.0) };
    // four edges
    concepts::CircleMappingEdge2d edges[4] = 
      {concepts::CircleMappingEdge2d(2.0, vtx[0], vtx[1]),
       concepts::CircleMappingEdge2d(3.0, vtx[1], vtx[2]),
       concepts::CircleMappingEdge2d(0.0, vtx[2], vtx[3]),
       concepts::CircleMappingEdge2d(-2.0, vtx[3], vtx[0])};

    // Coordinates on the edges
    for(uint j=0;j<4;++j) {    
      std::cout << "X{" << j+1 << "} = [";
      for(uint i=0;i<=n;++i) {
	Real t = 1.0/n*i;
	Real2d x(edges[j](t));
	std::cout << "[" << x[0] << ";" << x[1] << "] ";
      }
      std::cout << "];" << std::endl;
    }
    // for plotting in matlab use
    // plot(X{1}(1,:), X{1}(2,:), '+');

    // Curved Quadrilaterial, edg1 is not given, so straight
    concepts::BlendingQuad2d blending(edges, edges + 1, 0, edges + 3);

    // output of element map (Coordinates in the Quadrilaterial)
    std::cout << "M = [";
    for(uint i=0; i<=n; ++i) {
      Real s = 1.0/n*i;
      for(uint j=0; j<=n; ++j) {
	Real t = 1.0/n*j;
	Real2d x(blending(s,t));
	std::cout << '[' << x[0] << ';' << x[1] << "] ";
      }
    }
    std::cout << "];" << std::endl;
    // for plotting in matlab use
    // plot(M(1,:), M(2,:), '+');

    // output of Jacobian in the Quadrilaterial
    std::cout << "J = [";
    for(uint i=0; i<=n; ++i) {
      Real s = 1.0/n*i;
      for(uint j=0; j<=n; ++j) {
	Real t = 1.0/n*j;
	concepts::MapReal2d J(blending.jacobian(s,t));
	std::cout << '[' << J(0,0) << ';' << J(1,0) << ';' << 
	  J(0,1) << ';' << J(1,1) << "] ";
      }
    }
    std::cout << "];" << std::endl;
    // for plotting in matlab use
    // quiver(M(1,:),M(2,:),J(1,:),J(2,:));
    // quiver(M(1,:),M(2,:),J(3,:),J(4,:));
  }
  catch(concepts::ExceptionBase& e) {
    std::cout << e << std::endl;
    return 1;
  }

  return 0;
}
