/* graphics of the data in Gnuplot
 */

#include <fstream>
#include "basics/exceptions.hh"
#include "basics/vectorsMatrices.hh"
#include "space/element.hh"
#include "space/tmatrix.hh"
#include "geometry/cell.hh"
#include "toolbox/elementGraphics.hh"
#include "dataGnuplot.hh"

// debuggig
#include "basics/debug.hh"

#define DGCappl_D 0

namespace graphics {

  // ******************************************************* DataGnuplotCell **

  template<typename F>
  DataGnuplotCell<F>::DataGnuplotCell
  (std::ofstream* ofs, const concepts::Vector<Real>& solution,
   const concepts::ElementFunction<Real>* fun) :
    ofs_(ofs), coeff_(0), solution_(&solution), frm_(0), frmElm_(0),
    fun_(fun) {}

  template<typename F>
  DataGnuplotCell<F>::DataGnuplotCell
  (std::ofstream* ofs, const concepts::Formula& frm,
   const concepts::ElementFunction<Real>* fun) :
    ofs_(ofs), coeff_(0), solution_(0), frm_(frm.clone()), frmElm_(0),
    fun_(0) {}

  template<typename F>
  DataGnuplotCell<F>::DataGnuplotCell
  (std::ofstream* ofs, const concepts::ElementFormula<F>& frm,
   const concepts::ElementFunction<Real>* fun) :
    ofs_(ofs), coeff_(0), solution_(0), frm_(0), frmElm_(&frm), fun_(0) {}

  template<typename F>
  DataGnuplotCell<F>::~DataGnuplotCell() {
  }

  template<typename F>
  void DataGnuplotCell<F>::operator() (const concepts::Element<F>& elm)
    throw(concepts::MissingFeature) {
    DEBUGL(DGCappl_D, elm);
    if (solution_)
      elm.T().extract(*solution_, coeff_);
    elm.graphics()->operator()(elm,
			       concepts::ElementGraphics<F>::DATA_GNUPLOT,
			       *this);
  }

  template<typename F>
  void DataGnuplotCell<F>::operator() (const concepts::Cell&)
    throw(concepts::MissingFeature) {
    throw conceptsException(concepts::MissingFeature("cell not supported"));
  }

  template class DataGnuplotCell<Real>;

} // namespace graphics
