/* Linear form for hp 1D FEM
 */

#ifndef linearForm1D_hh
#define linearForm1D_hh

#include <memory>
#include "basics/typedefs.hh"
#include "function/linearForm.hh"

namespace concepts {
  // forward declarations
  class Formula;
  class BoundaryConditions;
}

namespace hp1D {
  using concepts::Real;

  // ***************************************************************** Riesz **

  /** Linear form in 3D.

      This linear form computes
      \f[ \int_K f v \, dx. \f]
      @author Philipp Frauenfelder, 2002
  */
  class Riesz : public concepts::LinearForm<Real> {
  public:
    /** Constructor. Parses the formula.
	@param frm The formula
	@param bc Boundary conditions, defaults to homogeneous
    */
    Riesz(const concepts::Formula& frm, concepts::BoundaryConditions* bc = 0); 
    virtual ~Riesz();

    /** Computes the element load vector. As for the computation of an
	element stiffness matrix, there are the loops over all quadrature
	points and the loops over all shape functions.
	@param elm The element for which the load vector should be computed.
	@param em The load vector
    */
    void operator()(const concepts::Element<Real>& elm,
		    concepts::ElementMatrix<Real>& em);
  protected:
    virtual std::ostream& info(std::ostream& os) const;
  private:
    /// Formula
    std::auto_ptr<const concepts::Formula> frm_;

    /// Reference to the boundary conditions
    concepts::BoundaryConditions* bc_;
  }; 

} // namespace hp1D

#endif // linearForm1D_hh
