/* a priori refinement for quads
 */

#include "aprioriRef2D.hh"
#include "hp2D/quad.hh"
#include "basics/exceptions.hh"
#include "geometry/topology.hh"

// debugging
#define APriori2DAppl_D 0

namespace hp2D {

  // ***************************************************** APrioriRefinement **

  void APrioriRefinement::operator() (const concepts::Element<Real>& elm)
    throw(concepts::MissingFeature) {

    const Quad* quad = dynamic_cast<const Quad*>(&elm);
    if (quad) {
      spc_.adjust(elm, computeRef_(quad->support(), quad->cell().level()));
      return;
    } // if quad

    throw conceptsException(concepts::MissingFeature("element not supported"));
  }

  std::ostream& APrioriRefinement::info(std::ostream& os) const {
    os << "hp2D::APrioriRefinement(";
    if  (attribVtx_) os << "vtx = " << attribVtx_ << ", ";
    if (attribEdge_) os << "edge = " << attribEdge_ << ", ";
    if (level_.l_[0] < MAXSHORT) os << "neccessary level " << level_ << ", ";
    return os << "pmax = (" << p_[0] << ", " << p_[1] << "))";
  }

  concepts::AdaptiveAdjustP<2> APrioriRefinement::computeRef_
  (const concepts::Quad& q, concepts::Level<2> level) const {
    concepts::Level<2> l(0);
    int p[2] = { p_[0], p_[1] };

    if (attribVtx_) {
      concepts::Connector0* vtx = 0;
      for (uint i = 0; (vtx = q.vertex(i)) != 0; ++i) {
	if (vtx->attrib() == attribVtx_) {
	  if (subdiv_ & X) l.l_[0] = 1;
	  if (subdiv_ & Y) l.l_[1] = 1;
	}
      } // for vertices
    }
    if (attribEdge_) {
      concepts::Connector1* edge = 0;
      for (uint i = 0;
	   (edge = q.edge(i)) != 0; ++i) {
	if (edge->attrib() == attribEdge_)
	  switch (i) {
	  case 1: case 3: if (subdiv_ & X) l.l_[0] = 1; break;
	  case 0: case 2: if (subdiv_ & Y) l.l_[1] = 1; break;
	  default:
	    throw conceptsException(concepts::ExceptionBase());
	  }
      } // for edges
    }

    DEBUGL(APriori2DAppl_D, "level neccessary = " << level_ << ", level = " << level);

    if ((level_.l_[0] <= level.l_[0]) & (level_.l_[1] <= level.l_[1])) {
      if (subdiv_ & X) l.l_[0] = 1;
      if (subdiv_ & Y) l.l_[1] = 1;
    }

    for (uint i = 0; i < 2; ++i) {
      if (l.l_[i] != 0) p[i] = 0;
    }
    DEBUGL(APriori2DAppl_D, "refining " << q << " with " << l <<
	   " and p = (" << p[0] << "," << p[1] << ")");
    return concepts::AdaptiveAdjustP<2>(l, p);
  }

} // namespace hp2D
