/** @file buildEdgeDofs.hh Factored out part of building elements in
    hp2D::Space.
    @author Philipp Frauenfelder, 2004
*/

#ifndef buildEdgeDofs2D_hh
#define buildEdgeDofs2D_hh

#include "hp2D/space.hh"
#include "hp2D/buildDofsBase.hh"

namespace hp2D {

  // forward declaration
  class Space;

  // ********************************************************* BuildEdgeDofs **

  /** Responsible to build the edge degrees of freedom in a space.
      A call to the application operator builds the T columns for the
      edges of a given quad.

      If you want to change the way this is done, derive from this
      class and overwrite the application operator. The access to the
      internal data of the space (which is needed) is provided through
      the protected member functions of BuildDofsBase (it is a friend
      of Space).

      This class is registered as the default strategy in a Space to
      build the edge degrees of freedom. Giving Space a different
      class through the use of Space::buildEdgeDofs, you can change
      that behaviour.

      @author Philipp Frauenfelder, 2004
  */
  class BuildEdgeDofs : public BuildDofsBase {
  public:
    /// Default constructor
    BuildEdgeDofs() {}
    /** Constructor.
	@param spc Space to build the edge degrees of freedom from
    */
    BuildEdgeDofs(Space& spc) : BuildDofsBase(spc) {}
    virtual void operator()(concepts::Quad& cntr,
			    concepts::TColumn<Real>*& T1);
    virtual BuildEdgeDofs* clone(Space* spc = 0) const;
  };

} // namespace hp2D

#endif // buildEdgeDofs2D_hh
