/** @file buildVtxDofs.hh Factored out part of building elements in
    hp2D::Space.
    @author Philipp Frauenfelder, 2004
*/

#ifndef buildVtxDofs2D_hh
#define buildVtxDofs2D_hh

#include "hp2D/space.hh"
#include "hp2D/buildDofsBase.hh"

namespace hp2D {

  // ******************************************************* BuildVertexDofs **

  /** Responsible to build the vertex degrees of freedom in a space.
      A call to the application operator builds the T columns for the
      vertices of a given quad.

      If you want to change the way this is done, derive from this
      class and overwrite the application operator. The access to the
      internal data of the space (which is needed) is provided through
      the protected member functions of BuildDofsBase (it is a friend
      of Space).

      This class is registered as the default strategy in a Space to
      build the vertex degrees of freedom. Giving Space a different
      class through the use of Space::buildVertexDofs, you can change
      that behaviour.

      @author Philipp Frauenfelder, 2004
  */
  class BuildVertexDofs : public BuildDofsBase {
  public:
    /// Default constructor
    BuildVertexDofs() {}
    /** Constructor.
	@param spc Space to build the vertex degrees of freedom from
    */
    BuildVertexDofs(Space& spc) : BuildDofsBase(spc) {}
    virtual void operator()(concepts::Quad& cntr,
			    concepts::TColumn<Real>*& T1);
    virtual BuildVertexDofs* clone(Space* spc = 0) const;
  };

} // namespace hp2D

#endif // buildVtxDofs2D_hh
