#include "hp2D/space.hh"
#include "basics/exceptions.hh"

// debugging
#include "basics/debug.hh"

#define createCellList_D 0
#define createVtxEdgFaceList_D 0

namespace hp2D {

  // ***************************************************************** Space **

  void Space::createCellList_(const concepts::Connector& cntr,
			      const concepts::CellData* father) {
    concepts::AdaptiveControlP<2>& ctrl2(ctrl2_[cntr.key()]);

    std::pair<std::map<uint, concepts::CellData>::iterator, bool> i =
      cellList_.insert
      (std::pair<uint, concepts::CellData>
       (cntr.key().key(), concepts::CellData(cntr, father)));
    father = &(i.first->second);
    DEBUGL(createCellList_D, "inserted " << *father);
    if (! ctrl2.tag_.isMember()) {
      const concepts::Connector* chld = 0;
      for(uint i = 0; (chld = cntr.child(i)) != 0; ++i) {
	createCellList_(*chld, father);
      } // for child
    }
  }

  void Space::createVtxEdgList_(const concepts::Connector2& cntr) {
    concepts::AdaptiveControlP<2>& ctrl2(ctrl2_[cntr.key()]);
    if (! ctrl2.tag_.isMember()) {
      const concepts::Connector2* chld = 0;
      for(uint i = 0; (chld = cntr.child(i)) != 0; ++i) {
	createVtxEdgList_(*chld);
      } // for child
    } else {
      std::map<uint, concepts::CellData>::iterator i =
	cellList_.find(cntr.key().key());
      conceptsAssert(i != cellList_.end(), concepts::Assertion());
      DEBUGL(createVtxEdgFaceList_D, i->second);

      // insert vertices into vertexList_
      const concepts::Connector1* cntr1 = 0;
      for(uint k = 0; (cntr1 = cntr.edge(k)) != 0; ++k) { // loop: edges
	std::map<uint, concepts::EdgeData>::iterator j =
	  edgeList_.find(cntr1->key().key());
	if (j == edgeList_.end()) {
	  j = edgeList_.insert
	    (std::pair<uint, concepts::EdgeData>
	     (cntr1->key().key(), concepts::EdgeData(*cntr1))).first;
	  DEBUGL(createVtxEdgFaceList_D, "registered " << j->second);
	} else
	  DEBUGL(createVtxEdgFaceList_D, "found " << j->second);
	j->second.addCell(i->second);

	const concepts::Connector0* cntr0 = 0;
	for(uint m = 0; (cntr0 = cntr1->vertex(m)) != 0; ++m) {
	  std::map<uint, concepts::VertexData>::iterator l =
	    vertexList_.find(cntr0->key().key());
	  if (l == vertexList_.end())
	    l = vertexList_.insert
	      (std::pair<uint, concepts::VertexData>
	       (cntr0->key().key(), concepts::VertexData(*cntr0))).first;
	  l->second.addCell(i->second);
	  l->second.addEdge(j->second);
	} // for vertices
      } // for edges
    } // cell is member of space
  }

} // namespace hp2D
