/* weight functions for weighted regularization
 */

#if __GNUC__ == 2
#  include <float.h>
#  define MAX DBL_MAX
#else
#  include <limits>
#  define MAX std::numeric_limits<Real>::max()
#endif
#include "hp2D/shortestDist.hh"
#include "hp2D/singularSet.hh"
#include "hp2D/quad.hh"

// debugging
#include "basics/debug.hh"

#define hpShortestDistAppl_D 0

namespace hp2D {

  std::ostream& operator<<(std::ostream& os, const TrivialWeight& p) {
    return os << "TrivialWeight";
  }

  std::ostream& operator<<(std::ostream& os, const Postprocess4& p) {
    return os << "Postprocess4";
  }

  std::ostream& operator<<(std::ostream& os, const Postprocess7& p) {
    return os << "Postprocess7";
  }

  std::ostream& operator<<(std::ostream& os, const Postprocess8& p) {
    return os << "Postprocess8";
  }

  std::ostream& operator<<(std::ostream& os, const Postprocess9& p) {
    return os << "Postprocess9";
  }

  std::ostream& operator<<(std::ostream& os, const PostprocessSqrt& p) {
    return os << "PostprocessSqrt";
  }

  // ********************************************************** ShortestDist **

  Real ShortestDist::operator()(const SingularSet& singularities,
				concepts::Real2d point,
				const Quad* elm) const {
    DEBUGL(hpShortestDistAppl_D, "singularities = " << singularities);
    DEBUGL(hpShortestDistAppl_D, "point = " << point);
    DEBUGL(hpShortestDistAppl_D, "elm = " << *elm);

    Real shortest = MAX;
    uint anz = 0; // number of singularities

    // scanner over all singular vertices
    std::auto_ptr<concepts::Scan<SingularVertex> > sc
      (singularities.verticesScan());
    while (*sc) {
      SingularVertex& singVtx = (*sc)++;
      Real dist = singVtx.distance(point, elm);
      DEBUGL(hpShortestDistAppl_D, "dist^2 = " << dist << " from " << point
	     << " to " << singVtx);
      if (dist < shortest) shortest = dist;
      ++anz;
    }

    // without marked singular edges or vertices is squared distance 1.0
    if (anz==0) shortest = 1.0;

    DEBUGL(hpShortestDistAppl_D, "shortest dist^2 = " << shortest);

    return shortest;
  }

  std::ostream& operator<<(std::ostream& os, const ShortestDist& p) {
    return os << "ShortestDist";
  }

} // namespace hp2D
