/* singular vertices
 */

#include "hp2D/singularSet.hh"
#include "hp2D/quad.hh"
#include "toolbox/dynArray.hh"

// debugging
#include "basics/debug.hh"

#define singularVtxDist_D 0

#define hpSingularSetAppl_D 0
#define hpSingularSetApplArray_D 0
#define hpSingularSetApplLists_D 0

namespace hp2D {

  // ******************************************************** SingularVertex **

  std::ostream& operator<<(std::ostream& os, const SingularVertex& s) {
    return os << "SingularVertex(Vertex = " << *(s.vertex_)
	      << ", vtx0 = " << s.vtx0_ << ")";
  }

  Real SingularVertex::distance(concepts::Real2d point) const {
    concepts::Real2d diffVector = point - vtx0_;
    DEBUGL(singularVtxDist_D, "dist^2 between " << vtx0_ << " and " << point
	   << " is " << diffVector.l2_2());
    return diffVector.l2_2();
  }

  // *********************************************************** SingularSet **

  std::ostream& operator<<(std::ostream& os, const SingularSet& s) {
    os << "SingularSet(";
    std::auto_ptr<concepts::Scan<SingularVertex> > sc(s.verticesScan());
    while (*sc) {
      SingularVertex& singVtx = (*sc)++;
      os << singVtx;
      if (*sc) os << ", ";
    }
    return os << ')';
  }

  SingularSet::~SingularSet() {
    concepts::Joiner<SingularVertex*, 1>::destructor(vertices_); 
  }

  void SingularSet::add(const concepts::Attribute& attrib,
			concepts::Mesh2& msh) {
    DEBUGL(hpSingularSetAppl_D, "attrib = " << attrib << " in mesh " << msh);

    // arrays for managing the marking process
    // coding for vertices: , 0 - checked, not to mark, 1 - not yet checked,
    //                        2 - checked, to mark
    concepts::DynArray<uint> singVertMap(2, 3, 1);

    // scanner over all already marked vertices or edges
    std::auto_ptr<concepts::Scan<SingularVertex> > scv(verticesScan());

    // collect already marked vertices and edges in the arrays
    while (*scv) {
      const SingularVertex& singVtx = (*scv)++;
      singVertMap[singVtx.vertex()->key()] = 0;
    }

    DEBUGL(hpSingularSetApplArray_D, "VertMap = " << singVertMap);

    // mark vertices
    std::auto_ptr<concepts::Scan2> sc(msh.scan());
    while (*sc) {
      concepts::Cell2& cell = (*sc)++;
      concepts::Connector0* vtx;
      for (uint j = 0; (vtx = cell.connector().vertex(j)) != 0; ++j)
	if ((singVertMap[vtx->key()] == 1) && (vtx->attrib() == attrib)) {
	  // add Vertex in the list of singular Vertices
	  vertices_ = new concepts::Joiner<SingularVertex*, 1>
		(new SingularVertex(vtx, cell.vertex(j)), vertices_);
	  DEBUGL(hpSingularSetAppl_D, "singular marked: " << *vtx << "-"
		 << cell.vertex(j));
	  singVertMap[vtx->key()] = 0;
      }
    }

    DEBUGL(hpSingularSetApplArray_D, "VertMap = " << singVertMap);

#if hpSingularSetApplLists_D
    DEBUGL(hpSingularSetApplLists_D, "singular vertices:");
    {
      std::auto_ptr<concepts::Scan<SingularVertex> > sc
	(verticesScan());
      while (*sc) {
	const SingularVertex& singVtx = (*sc)++;
	DEBUGL(hpSingularSetApplLists_D, singVtx);
      }
    }
#endif
  }

} // namespace hp2D
