/* Testsuite for 2D hp-FEM space
 */

#include <memory>
#include "basics/exceptions.hh"
#include "geometry.hh"
#include "hp2D/space.hh"
#include "hp2D/aprioriRef2D.hh"
#include "spaceTest.hh"

namespace test {

  // *********************************************************** SpaceTest2D **

  void SpaceTest2D::run() {
    testBuild(); testCopy();
  }

  void SpaceTest2D::testBuild() {
    concepts::Import2dMesh m
      (SOURCEDIR "/geometry/testsuite/Coordinates2D.dat",
       SOURCEDIR "/geometry/testsuite/quad.dat",
       SOURCEDIR "/geometry/testsuite/empty.dat");
    _test(m.ncell() == 1);
    hp2D::Space spc(m, 0, 1);
    const hp2D::Space& spc_c = spc;
    try {
      spc_c.dim();
      _fail("Should have thrown exception!");
    }
    catch (concepts::SpaceNotBuilt& e) {
      _succeed();
    }
    spc.rebuild();
    try {
      spc_c.dim();
      _succeed();
    }
    catch (concepts::SpaceNotBuilt& e) {
      _fail("Should not have thrown exception!");
    }
  }

  void SpaceTest2D::testCopy() {
    concepts::Import2dMesh m
      (SOURCEDIR "/geometry/testsuite/Coordinates2D.dat",
       SOURCEDIR "/geometry/testsuite/quad.dat",
       SOURCEDIR "/geometry/testsuite/empty.dat");
    _test(m.ncell() == 1);
    hp2D::Space spc(m, 0, 1);
    hp2D::Space spc2 = spc.rebuild();
    _test(spc.nelm() == 1);
    spc2.rebuild();
    _test(spc2.nelm() == 1);
    int pMax[2] = { 1, 1 };
    hp2D::APrioriRefinement refineSpace(spc, 0, 0, pMax);
    concepts::GlobalPostprocess<Real> post(spc);
    post(refineSpace);
    _test(spc.dim() > spc2.dim());
    _test(spc.nelm() == spc2.nelm());
    std::auto_ptr<hp2D::Space::Scan> sc(spc.scan()), sc2(spc2.scan());
    if (*sc && *sc2) {
      hp2D::Element<Real>& e1 = (*sc)++, & e2 = (*sc2)++;
      _test(&e1 != &e2);
      _test(e1.support() == e2.support());
    } else {
      _fail("scanners should be available");
    }

    {
      hp2D::Space spc3 = spc;
    }
    spc.rebuild();
  }

} // namespace test

int main(int, char**) {
  concepts::Stacktrace::doit() = false;
  try {
    test::SpaceTest2D t;
    t.run();
    return t.report();
  }
  catch (concepts::ExceptionBase& e) {
    std::cout << e << std::endl;
    return 1;
  }
}
