/* Factored out part of building elements in hp3D::Space.
 */

#include "buildFaceDofs.hh"
#include "hexFunctions.hh"

// debugging
#include "basics/debug.hh"

#define hpBuildFaceDofsAppl_D 0
#define hpSpace3Ddof_D 0

namespace hp3D {

  // ********************************************************* BuildFaceDofs **

  void BuildFaceDofs::operator()(concepts::Hexahedron& cntr,
				 concepts::TColumn<Real>*& T1) {
    conceptsAssert(spc_, concepts::Assertion());
    const concepts::AdaptiveControlP<3>& ctrl3(ctrl3_(cntr.key()));
    concepts::TColumnTensor<Real, 3>* TTensor = 0;
    uint ll[3] = { 0, 0, 0 };
    ushort Pmax[3] = { 0, 0, 0 };
    computePmax_(cntr, Pmax);
    uint nn[3] = { Pmax[0]+1, Pmax[1]+1, Pmax[2]+1 };

    // loop over faces
    concepts::Connector2* cntr2 = 0;
    for (uint i = 0; (cntr2 = cntr.face(i)) != 0; ++i) {
      concepts::AdaptiveControlP<2>& ctrl2(ctrl2_(cntr2->key()));
      if (ctrl2.tag_.isMember() || ctrl2.tag_.isPassive()) { 
	std::map<uint, concepts::FaceData>::const_iterator j =
	  faceListFind(cntr2->key().key());
	if (ctrl2.tag_.isMember() && ctrl2.tag_.isActive() &&
	    (j != faceListEnd()) && (j->second.hasCell(cntr))) {
	  // active member
	  DEBUGL(hpBuildFaceDofsAppl_D, cntr.key() << " - treating face "
		 << i << " = " << *cntr2 << ", ctrl = " << ctrl2);
	  if ((ctrl2.p_[0] > 1) && (ctrl2.p_[1] > 1)) {
	    // has dofs
	    int p0_1 = ctrl2.p_[0] - 1;
	    int p1_1 = ctrl2.p_[1] - 1;
	    bool addToDim = false;
	    if (! ctrl2.tag_.idxValid()) {
	      // not yet counted
	      ctrl2.idx_ = dim_();
	      addToDim = true;
	      ctrl2.tag_.validateIdx();
	      DEBUGL(hpSpace3Ddof_D, *cntr2 << " has dof " << ctrl2.idx_);
	    }

	    // check if signs need to be flipped
	    bool flip0, flip1;
	    HexFunctions::faceShapefctOrientation(cntr, i, flip0, flip1);
// 	    DEBUGL(hpBuildFaceDofsAppl_D, cntr.key() << " - flip0 = "
// 		   << flip0 << ", flip1 = " << flip1);

	    // get as much index information as possible
	    uint pIndex = 0, qIndex = 0, rIndex = 0;
	    HexFunctions::faceOrientation(cntr, i, pIndex, qIndex, rIndex);
	    HexFunctions::faceIndex(i, rIndex, ll);
	    conceptsAssert(pIndex != rIndex, concepts::Assertion());
	    conceptsAssert(qIndex != rIndex, concepts::Assertion());
	    conceptsAssert(pIndex != qIndex, concepts::Assertion());

	    // assume a quad as face
	    unsigned int idx = ctrl2.idx_;
	    for (int p = 0; p < p0_1; ++p) {
	      for (int q = 0; q < p1_1; ++q) {
		if (trunk(p, q, ctrl2.p_)) {
		  if (addToDim) ++dim_();
		  T1 = TTensor
		    = new concepts::TColumnTensor<Real, 3>(nn, idx++, T1);

		  ll[pIndex] = p + 2;
		  ll[qIndex] = q + 2;

		  // sign of the shape function
		  Real s = 1.0;
		  if ( ( ((int)flip0)*p + ((int)flip1)*q ) % 2 )
		    s = -1.0;

		  (*TTensor)[ll] = s;
		  DEBUGL(hpBuildFaceDofsAppl_D, cntr.key()
			 << " - active dof: ll = (" << ll[0] << ", "
			 << ll[1] << ", " << ll[2] << "), p = (" << ctrl2.p_[0]
			 << ", " << ctrl2.p_[1] << "), idx = " << ctrl2.idx_
			 << ", s = " << s);
 		} // if "trunk"
	      } // for q
	    } // for p
	  } // p > 1
	} // active member
	if (! ctrl3.tag_.isMember()) deactivate_(*cntr2);
      }

      uint pIndex = 0, qIndex = 0, rIndex = 0;
      HexFunctions::faceOrientation(cntr, i, pIndex, qIndex, rIndex);
    } // for i
  }

  BuildFaceDofs* BuildFaceDofs::clone(Space* spc) const {
    if (spc) return new BuildFaceDofs(*spc);
    else return new BuildFaceDofs(*spc_);
  }

  bool BuildFaceDofs::trunk(uint p, uint q, const ushort P[2]) {
    return true;
  }

  // ************************************************* BuildFaceDofsLinTrunk **

  BuildFaceDofsLinTrunk* BuildFaceDofsLinTrunk::clone(Space* spc) const {
    if (spc) return new BuildFaceDofsLinTrunk(*spc);
    else return new BuildFaceDofsLinTrunk(*spc_);
  }

  bool BuildFaceDofsLinTrunk::trunk(uint p, uint q, const ushort P[2]) {
    return ((p + q) <= ((P[0] > P[1]) ? P[0] : P[1]));
  }

} // namespace hp3D
