/* Factored out part of building elements in hp3D::Space.
 */

#include "buildInnerDofs.hh"
#include "hexFunctions.hh"

// debugging
#include "basics/debug.hh"

#define hpBuildInnerDofsAppl_D 0
#define hpBuildInnerDofsLinTrunk_D 0
#define hpSpace3Ddof_D 0

namespace hp3D {

  // ******************************************************** BuildInnerDofs **

  void BuildInnerDofs::operator()(concepts::Hexahedron& cntr,
				   concepts::TColumn<Real>*& T1) {
    conceptsAssert(spc_, concepts::Assertion());
    concepts::AdaptiveControlP<3>& ctrl3(ctrl3_(cntr.key()));
    concepts::TColumnTensor<Real, 3>* TTensor = 0;
    uint ll[3] = { 0, 0, 0 };
    ushort Pmax[3] = { 0, 0, 0 };
    computePmax_(cntr, Pmax);
    uint nn[3] = { Pmax[0]+1, Pmax[1]+1, Pmax[2]+1 };

    if ( (ctrl3.p_[0] > 1) && (ctrl3.p_[1] > 1) && (ctrl3.p_[2] > 1) ) { // p int
      ctrl3.idx_ = dim_();
      ctrl3.tag_.validateIdx();
      DEBUGL(hpSpace3Ddof_D, cntr << " has dof " << ctrl3.idx_);

      uint idx = ctrl3.idx_;
      for (uint& i = ll[0] = 2; i < (uint)ctrl3.p_[0]+1; ++i) { // p int
	for (uint& j = ll[1] = 2; j < (uint)ctrl3.p_[1]+1; ++j) { // p int
	  for (uint& l = ll[2] = 2; l < (uint)ctrl3.p_[2]+1; ++l) { // p int
	    if (trunk(i, j, l, ctrl3.p_)) {
	      ++dim_();
	      T1 = TTensor
		= new concepts::TColumnTensor<Real, 3>(nn, idx++, T1);
	      (*TTensor)[ll] = 1.0;
	      DEBUGL(hpBuildInnerDofsAppl_D, cntr.key()
		     << " - active dof: ll = (" << ll[0]
		     << ", " << ll[1] << ", " << ll[2] << ')');
 	    } // if "trunk"
	  } // for l
	} // for j
      } // for i
    }
  }

 BuildInnerDofs* BuildInnerDofs::clone(Space* spc) const {
   if (spc) return new BuildInnerDofs(*spc);
   else return new BuildInnerDofs(*spc_);
 }

  bool BuildInnerDofs::trunk(uint p, uint q, uint r, const ushort P[3]) {
    return true;
  }

  // ************************************************ BuildInnerDofsLinTrunk **

 BuildInnerDofsLinTrunk* BuildInnerDofsLinTrunk::clone(Space* spc) const {
   if (spc) return new BuildInnerDofsLinTrunk(*spc);
   else return new BuildInnerDofsLinTrunk(*spc_);
 }

  bool BuildInnerDofsLinTrunk::trunk(uint p, uint q, uint r,
				     const ushort P[3]) {
    uint PintMax = P[0] > P[1] ? P[0] : P[1];
    PintMax = PintMax > P[2] ? PintMax : P[2];
    DEBUGL(hpBuildInnerDofsLinTrunk_D, PintMax << " is max of (" << P[0]
	   << ", " << P[1] << ", " << P[2] << ')');
    return ((p + q + r) <= (PintMax + 1));
  }

} // namespace hp3D
