/* Factored out part of building elements in hp3D::Space.
 */

#include "buildVtxDofs.hh"
#include "hexFunctions.hh"

// debugging
#include "basics/debug.hh"

#define hpBuildVtxDofsAppl_D 0
#define hpSpace3Ddof_D 0

namespace hp3D {

  // ******************************************************* BuildVertexDofs **

  void BuildVertexDofs::operator()(concepts::Hexahedron& cntr,
				   concepts::TColumn<Real>*& T1) {
    conceptsAssert(spc_, concepts::Assertion());
    const concepts::AdaptiveControlP<3>& ctrl3(ctrl3_(cntr.key()));
    concepts::TColumnTensor<Real, 3>* TTensor = 0;
    uint ll[3] = { 0, 0, 0 };
    ushort Pmax[3] = { 0, 0, 0 };
    computePmax_(cntr, Pmax);
    uint nn[3] = { Pmax[0]+1, Pmax[1]+1, Pmax[2]+1 };

    // loop over vertices
    concepts::Vertex* cntr0 = 0;
    for(uint i = 0; (cntr0 = cntr.vertex(i)) != 0; ++i) {
      concepts::AdaptiveControl& ctrl0(ctrl0_(cntr0->key()));
      DEBUGL(hpBuildVtxDofsAppl_D, cntr.key() << " - " << *cntr0
	     << ", tag = " << ctrl0.tag_);
      if (ctrl0.tag_.isMember() || ctrl0.tag_.isPassive()) {
	// is member of the space or passive
	std::map<uint, concepts::VertexData>::const_iterator j =
	  vertexListFind(cntr0->key().key());
	if (ctrl0.tag_.isMember() && ctrl0.tag_.isActive() &&
	    (j != vertexListEnd()) && (j->second.hasCell(cntr))) {
	  // active member of the space
	  if (! ctrl0.tag_.idxValid()) { 
	    // not yet counted
	    ctrl0.idx_ = dim_(); ++dim_();
	    ctrl0.tag_.validateIdx();
	    DEBUGL(hpSpace3Ddof_D, *cntr0 << " has dof " << ctrl0.idx_);
	  }
	  T1 = TTensor
	    = new concepts::TColumnTensor<Real, 3>(nn, ctrl0.idx_, T1);

	  HexFunctions::vertexIndex(i, ll);
	  conceptsAssert(ll[0] < 2, concepts::Assertion());
	  conceptsAssert(ll[1] < 2, concepts::Assertion());
	  conceptsAssert(ll[2] < 2, concepts::Assertion());

 	  DEBUGL(hpBuildVtxDofsAppl_D, cntr.key() << " - active dof: ll = ("
		 << ll[0] << ", " << ll[1] << ", " << ll[2] << "), vtx = "
		 << cntr0->key() << ", idx = " << ctrl0.idx_);
	  (*TTensor)[ll] = 1.0;
	} // if member of the space and active
	if (! ctrl3.tag_.isMember()) deactivate_(*cntr0);
      } // if member of the space or passive
    } // for vertices    
  }

  BuildVertexDofs* BuildVertexDofs::clone(Space* spc) const {
    if (spc) return new BuildVertexDofs(*spc);
    else return new BuildVertexDofs(*spc_);
  }

} // namespace hp3D
