/** @file buildVtxDofs.hh Factored out part of building elements in
    hp3D::Space.
    @author Philipp Frauenfelder, 2004
*/

#ifndef buildVtxDofs_hh
#define buildVtxDofs_hh

#include "hp3D/space.hh"
#include "hp3D/buildDofsBase.hh"

namespace hp3D {

  // forward declaration
  class Space;

  // ******************************************************* BuildVertexDofs **

  /** Responsible to build the vertex degrees of freedom in a space.
      A call to the application operator builds the T columns for the
      vertices of a given hexahedron.

      If you want to change the way this is done, derive from this
      class and overwrite the application operator. The access to the
      internal data of the space (which is needed) is provided through
      the protected member functions of BuildDofsBase (it is a friend
      of Space).

      This class is registered as the default strategy in a Space to
      build the vertex degrees of freedom. Giving Space a different
      class through the use of Space::buildVertexDofs, you can change
      that behaviour.

      @author Philipp Frauenfelder, 2004
  */
  class BuildVertexDofs : public BuildDofsBase {
  public:
    /// Default constructor
    BuildVertexDofs() {}
    /** Constructor.
	@param spc Space to build the vertex degrees of freedom from
    */
    BuildVertexDofs(Space& spc) : BuildDofsBase(spc) {}
    virtual void operator()(concepts::Hexahedron& cntr,
			    concepts::TColumn<Real>*& T1);
    virtual BuildVertexDofs* clone(Space* spc = 0) const;
  };

} // namespace hp3D

#endif // buildVtxDofs_hh
