#include "hp3D/space.hh"
#include "basics/exceptions.hh"
#include "basics/debug.hh"

// debugging

#define createCellList_D 0
#define createVtxEdgFaceList_D 0

namespace hp3D {

  // ***************************************************************** Space **

  void Space::createCellList_(const concepts::Connector& cntr,
			      const concepts::CellData* father) {
    concepts::AdaptiveControlP<3>& ctrl3(ctrl3_[cntr.key()]);

    // add cell to the list
    std::pair<std::map<uint, concepts::CellData>::iterator, bool> i =
      cellList_.insert
      (std::pair<uint, concepts::CellData>
       (cntr.key().key(), concepts::CellData(cntr, father)));

    // update father pointer
    father = &(i.first->second);

    // go to next level if we are not on the active level
    DEBUGL(createCellList_D, "inserted " << *father);
    if (! ctrl3.tag_.isMember()) {
      const concepts::Connector* chld = 0;
      for(uint i = 0; (chld = cntr.child(i)) != 0; ++i) {
	createCellList_(*chld, father);
      } // for child
    }
  }

  void Space::createEdgFaceList_(const concepts::Cell3& cell) {
    concepts::Connector3& cntr = cell.connector();
    concepts::AdaptiveControlP<3>& ctrl3(ctrl3_[cntr.key()]);
    if (! ctrl3.tag_.isMember()) {
      const concepts::Cell3* chld = 0;
      for(uint i = 0; (chld = cell.child(i)) != 0; ++i) {
	createEdgFaceList_(*chld);
      } // for child
    } else {
      std::map<uint, concepts::CellData>::iterator i =
	cellList_.find(cell.connector().key().key());
      conceptsAssert(i != cellList_.end(), concepts::Assertion());
      DEBUGL(createVtxEdgFaceList_D, i->second);

      // insert faces into faceList_
      const concepts::Connector2* cntr2 = 0;
      for(uint k = 0; (cntr2 = cntr.face(k)) != 0; ++k) { // loop over faces
	std::map<uint, concepts::FaceData>::iterator j =
	  faceList_.find(cntr2->key().key());
	if (j == faceList_.end()) {
	  j = faceList_.insert
	    (std::pair<uint, concepts::FaceData>
	     (cntr2->key().key(), concepts::FaceData(*cntr2))).first;
	  DEBUGL(createVtxEdgFaceList_D, "registered " << j->second);
	} else
	  DEBUGL(createVtxEdgFaceList_D, "found " << j->second);
	j->second.addCell(i->second);

	// insert edges into edgeList_
	const concepts::Connector1* cntr1 = 0;
	for(uint m = 0; (cntr1 = cntr2->edge(m)) != 0; ++m) { // loop: edges
	  std::map<uint, concepts::EdgeData>::iterator l =
	    edgeList_.find(cntr1->key().key());
	  if (l == edgeList_.end()) {
	    l = edgeList_.insert
	      (std::pair<uint, concepts::EdgeData>
	       (cntr1->key().key(), concepts::EdgeData(*cntr1))).first;
	    DEBUGL(createVtxEdgFaceList_D, "registered " << l->second);
	  } else
	    DEBUGL(createVtxEdgFaceList_D, "found " << l->second);
	  l->second.addCell(i->second);
	  l->second.addFace(j->second);
	} // for edges

      } // for faces
    } // cell is member of space
  }

  void Space::createVtxEdgList_(const concepts::Cell3& cell) {
    concepts::Connector3& cntr = cell.connector();
    concepts::AdaptiveControlP<3>& ctrl3(ctrl3_[cntr.key()]);
    if (! ctrl3.tag_.isMember()) {
      const concepts::Cell3* chld = 0;
      for(uint i = 0; (chld = cell.child(i)) != 0; ++i) {
	createVtxEdgList_(*chld);
      } // for child
    } else {
      std::map<uint, concepts::CellData>::iterator i =
	cellList_.find(cell.connector().key().key());
      conceptsAssert(i != cellList_.end(), concepts::Assertion());
      DEBUGL(createVtxEdgFaceList_D, i->second);

      // insert vertices into vertexList_
      const concepts::Connector1* cntr1 = 0;
      for(uint k = 0; (cntr1 = cntr.edge(k)) != 0; ++k) { // loop: edges
	std::map<uint, concepts::EdgeData>::iterator j =
	  edgeList_.find(cntr1->key().key());
	if (j == edgeList_.end()) {
	  j = edgeList_.insert
	    (std::pair<uint, concepts::EdgeData>
	     (cntr1->key().key(), concepts::EdgeData(*cntr1))).first;
	  DEBUGL(createVtxEdgFaceList_D, "registered " << j->second);
	} else
	  DEBUGL(createVtxEdgFaceList_D, "found " << j->second);
	j->second.addCell(i->second);

	const concepts::Connector0* cntr0 = 0;
	for(uint m = 0; (cntr0 = cntr1->vertex(m)) != 0; ++m) {
	  std::map<uint, concepts::VertexData>::iterator l =
	    vertexList_.find(cntr0->key().key());
	  if (l == vertexList_.end())
	    l = vertexList_.insert
	      (std::pair<uint, concepts::VertexData>
	       (cntr0->key().key(), concepts::VertexData(*cntr0))).first;
	  l->second.addCell(i->second);
	  l->second.addEdge(j->second);
	} // for vertices
      } // for edges
    } // cell is member of space
  }

} // namespace hp3D
