/* makes a function (formula) from a weight (can then be plotted)
 */

#include "formulaFromWeight.hh"
#include "hexahedron.hh"
#include "basics/exceptions.hh"

namespace hp3D {

  // ***************************************************** FormulaFromWeight **

  template<typename Weight>
  Real FormulaFromWeight<Weight>::operator()
    (const concepts::Element<Real>& elm, const Real p, const Real t) const {
    const Hexahedron* hex = dynamic_cast<const Hexahedron*>(&elm);
    conceptsAssert(hex, concepts::Assertion());
    return weight_(singularities_, concepts::Real3d(p, 0, 0), hex);
  }

  template<typename Weight>
  Real FormulaFromWeight<Weight>::operator()
    (const concepts::Element<Real>& elm, const concepts::Real2d& p,
     const Real t) const {
    const Hexahedron* hex = dynamic_cast<const Hexahedron*>(&elm);
    conceptsAssert(hex, concepts::Assertion());
    return weight_(singularities_, concepts::Real3d(p[0], p[1], 0), hex);
  }

  template<typename Weight>
  Real FormulaFromWeight<Weight>::operator()
    (const concepts::Element<Real>& elm, const concepts::Real3d& p,
     const Real t) const {
    const Hexahedron* hex = dynamic_cast<const Hexahedron*>(&elm);
    conceptsAssert(hex, concepts::Assertion());
    return weight_(singularities_, p, hex);
  }

  template class FormulaFromWeight<TrivialWeight>;
  template class FormulaFromWeight<ShortestDist>;
  template class FormulaFromWeight<ProductOfAll>;
  template class FormulaFromWeight<DaugeWeight>;

} // namespace hp3D
