/* makes a function (formula) from a weight (can then be plotted)
 */

#ifndef formulaFromWeight_hh
#define formulaFromWeight_hh

#include "space/element.hh"
#include "shortestDist.hh"

namespace hp3D {

  // ***************************************************** FormulaFromWeight **

  /** Makes it possible to plot a given \c Weight.
      @author Philipp Frauenfelder, 2003
      @ingroup maxwell_3d_wr
  */
  template<typename Weight = TrivialWeight>
  class FormulaFromWeight : public concepts::ElementFormula<Real> {
  public:
    /** Constructor.
	@param singularities Set of singularly marked edges and corners
    */
    FormulaFromWeight(const SingularSet& singularities) :
      weight_(), singularities_(singularities) {}

    virtual Real operator()(const concepts::Element<Real>& elm, const Real p,
			    const Real t = 0.0) const;
    virtual Real operator()(const concepts::Element<Real>& elm,
			    const concepts::Real2d& p,
			    const Real t = 0.0) const;
    virtual Real operator()(const concepts::Element<Real>& elm,
			    const concepts::Real3d& p,
			    const Real t = 0.0) const;
    virtual FormulaFromWeight<Weight>* clone() const {
      return new FormulaFromWeight(singularities_); }
  private:
    Weight weight_;
    const SingularSet& singularities_;
  };

} // namespace concepts

#endif // formulaFromWeight_hh
