/* space class for 3D hp FEM
 * enforceBC_
 */

#include "hp3D/space.hh"
#include "toolbox/boundary.hh"
#include "basics/debug.hh"

#define hpSpace3DRebuild0_D 0

namespace hp3D {

  // ***************************************************************** Space **

  void Space::enforceBC_(concepts::Hexahedron3d& cell) {
    DEBUGL(hpSpace3DRebuild0_D, "enforcing boundary conditions");
    conceptsAssert(bc_ != 0, concepts::Assertion());

    concepts::Connector3& cntr = cell.connector();

    // loop over faces
    concepts::Connector2* cntr2 = 0;
    for (uint i = 0; (cntr2 = cntr.face(i)) != 0; ++i) {
      if ((*bc_)(cntr2->attrib()).type() == concepts::Boundary::DIRICHLET) {
	DEBUGL(hpSpace3DRebuild0_D, *cntr2 << " has Dirichlet bc");
	// mark face as passive
	(*ctrl2_)[cntr2->key()].tag_.deactivate();

	// mark edges as passive
	concepts::Connector1* cntr1 = 0;
	for (uint j = 0; (cntr1 = cntr2->edge(j)) != 0; ++j) {
	  (*ctrl1_)[cntr1->key()].tag_.deactivate();
	}

	// mark vertices as passive
	concepts::Connector0* cntr0 = 0;
	for (uint j = 0; (cntr0 = cntr2->vertex(j)) != 0; ++j) {
	  ctrl0_[cntr0->key()].tag_.deactivate();
	}
      } // DIRICHLET bc
    } // for i

    // loop over edges
    concepts::Connector1* cntr1 = 0;
    for (uint i = 0; (cntr1 = cntr.edge(i)) != 0; ++i) {
      if ((*bc_)(cntr1->attrib()).type() == concepts::Boundary::DIRICHLET) {
	DEBUGL(hpSpace3DRebuild0_D, *cntr1 << " has Dirichlet bc");
	// mark edge as passive
	(*ctrl1_)[cntr1->key()].tag_.deactivate();

	// mark vertices as passive
	concepts::Connector0* cntr0 = 0;
	for (uint j = 0; (cntr0 = cntr1->vertex(j)) != 0; ++j) {
	  ctrl0_[cntr0->key()].tag_.deactivate();
	}
      } // DIRICHLET bc
    } // for i

    // loop over vertices
    concepts::Connector0* cntr0 = 0;
    for (uint i = 0; (cntr0 = cntr.vertex(i)) != 0; ++i) {
      if ((*bc_)(cntr0->attrib()).type() == concepts::Boundary::DIRICHLET) {
	DEBUGL(hpSpace3DRebuild0_D, *cntr0 << " has Dirichlet bc");
	// mark vertex as passive
	ctrl0_[cntr0->key()].tag_.deactivate();
      } // DIRICHLET bc
    } // for i
  }

}  // namespace hp3d
