/* zero tangential component, ie. u x n = 0
 */

#ifndef zerotangential3D_hh
#define zerotangential3D_hh

#include "constraints/analytical.hh"
#include "geometry/connector.hh"
#include "hp3D/hexahedron.hh"
#include "toolbox/dynArray.hh"

namespace hp3D {

  // *************************************************** ZeroTangentialValue **

  /** Sets the outer product of the coefficients and the normal vector
      to zero in every node on an edge with chosen attribute.
      @bug Does not work.
      @author Philipp Frauenfelder, 2002
  */
  template<typename F>
  class ZeroTangentialValue : public constraints::AnalyticalConstraint<F> {
  public:
    /** Constructor
	@param a Attribute to identify edges
    */
    ZeroTangentialValue(concepts::Attribute a) :
      attr_(a), scalElm_(0),
      ctrl0_(2, 10, (uint)-1), ctrl1_(2, 10, (uint)-1) {}
    ZeroTangentialValue(const ZeroTangentialValue& z) :
      attr_(z.attr_), scalElm_(z.scalElm_),
      ctrl0_(2, 10, (uint)-1), ctrl1_(2, 10, (uint)-1) {}
    virtual void lengthT(const concepts::Element<F>& e, uint& length);
    virtual void assembly(const concepts::Element<F>& e,
			  concepts::TColumn<F>*& t, uint& n, uint& j,
			  uint length);
    virtual constraints::AnalyticalConstraint<F>* clone() const;
    virtual void localMatrix(concepts::ElementMatrix<F>& em, uint& j,
			     const constraints::Element<F>& elm) const;
    virtual void localVector(concepts::ElementMatrix<F>& em, uint& j,
			     const constraints::Element<F>& elm) const;
  protected:
    virtual std::ostream& info(std::ostream& os) const;
  private:
    /// Topological entities with this attribute are considered
    concepts::Attribute attr_;
    mutable const Hexahedron* scalElm_;
    const concepts::Connector3&
    assertions_(const concepts::Element<F>& e) const;
    /// Vertices should only be considered once, store it here
    concepts::DynArray<uint> ctrl0_;
    /// Edges should only be considered once, store it here
    concepts::DynArray<uint> ctrl1_;
    void fillEM_(const uint ll[3], const ushort p[3], const uint offset,
		 uint& j, const concepts::Real3d e,
		 concepts::ElementMatrix<F>& em) const;
  };

} // namespace hp3D

#endif // zerotangential3D_hh
