/* Shape functions
 */

#ifndef shapefunction_hh
#define shapefunction_hh

#include "basics/outputOperator.hh"

namespace concepts {

  // ********************************************************* ShapeFunction **

  /** Abstract class for 1D shape function.

      The user can query how many form functions in how many points
      are available. The class is abstract since a derived class must
      give sense to the values_ array (ie. allocate and fill the entries).

      @author Philipp Fraunfelder, 2001.
   */
  template<class F>
  class ShapeFunction1D : public OutputOperator {
  public:
    ShapeFunction1D(const uint n, const uint nP) :
      values_(0), n_(n), nP_(nP) {}
    ~ShapeFunction1D() {}
    /// Returns the number of shape functions
    uint n() const { return n_; }
    /** Returns the number of abscissas (in which the shape functions
	are evaluated) */
    uint nP() const { return nP_; }
    /// Returns the values of the shape functions
    const F* values() const { return values_; }
  protected:
    virtual std::ostream& info(std::ostream& os) const = 0;

    /// Values of the shape functions
    F* values_;
  private:
    /// Number of shape functions
    uint n_;

    /// Number of points in which the shape functions are evaluated
    uint nP_;
  };

} // namespace concepts

#endif // shapefunction_hh
