// Spaces for constant and linear DG FEM in 2D

#include "geometry/mesh.hh"

#include "space.hh"

#include "basics/debug.hh"

#define dgSpace2dConstr_D 0

namespace linDG2D {

  // **************************************************************** Space **

  void Space::constructor_(concepts::Mesh2& msh) {
  
    // generate list of elements
    uint* idx = new uint[nDoF_()];
  
    std::auto_ptr<concepts::Scan2> sc(msh.scan());
    while (*sc) {
      concepts::Cell2& cell = (*sc)++;
      concepts::Triangle2d* cellT = dynamic_cast<concepts::Triangle2d*>(&cell);
      conceptsAssert(cellT, concepts::Assertion());
      for(uint i=0; i<nDoF_(); ++i) {
        idx[i] = dim_++;
      }
      nelm_++;
      elm_[cellT->connector().key()] = newElm_(*cellT, idx);
    }

    delete[] idx;
  }

  Space::~Space() {
    Map::iterator i = elm_.begin();
    while (!(i == elm_.end())) {
      delete i->second;
      i->second = 0;
      i++;
    }
  }

  std::ostream& Space::info(std::ostream& os) const {
    return os << "linDG2D::Space(dim = " << dim() 
              << ", nelm = " << nelm() << ')';
  }

  std::ostream& SpaceP1::info(std::ostream& os) const {
    return os << "linDG2D::SpaceP1(dim = " << dim() 
              << ", nelm = " << nelm() << ')';
  }

  std::ostream& SpaceP0::info(std::ostream& os) const {
    return os << "linDG2D::SpaceP0(dim = " << dim() 
              << ", nelm = " << nelm() << ')';
  }

} // namespace linDG2D
