// testing linDG2D::TriangleP1

#include "basics/testsuite.hh"
#include "basics/exceptions.hh"
#include "elmTest.hh"
#include "linDG2D/triangle.hh"

namespace test {

  using concepts::Real;

  // ****************************************************** DGTriangleP1Test **

  void DGTriangleP1Test::run() {
    concepts::Vertex v0, v1, v2;
    concepts::Edge e0(v0, v1);
    concepts::Edge e1(v1, v2);
    concepts::Edge e2(v2, v0);
    concepts::Triangle tco(e0, e1, e2);
    concepts::Real2d p0(-4.5, -3.0);
    concepts::Real2d p1(3.0, -1.5);
    concepts::Real2d p2(-1.5, 3.0);
    concepts::VertexTriangle2d tm(p0, p1, p2);
    concepts::Triangle2d tce(tco, tm);
    uint idx[3] = {0, 1, 2};
    linDG2D::TriangleP1 tel(tce, idx);

    try {
      testCell(tce);
      testShapeFct(tel);
      testShapeFctGradient(tel);
    }
    catch (concepts::ExceptionBase& e) {
      std::cout << e << std::endl;
      _fail("caught exception");
    }
  }

  void DGTriangleP1Test::testCell(concepts::Triangle2d& tce) {
    // test area
    _numtest(tce.area(), 20.25);
    //    _numtest(tce.diam(), 1.5*std::sqrt(26.));
    
    concepts::Triangle2d* tcech0 = tce.child(2);
    _numtest(tcech0->area(), 5.0625);
    //    _numtest(tcech0->diam(), 0.75*std::sqrt(26.));
    
    // test vertices
    _numtest(tce.vertex(0)[0], -4.5);
    _numtest(tce.vertex(0)[1], -3.0);
    _numtest(tce.vertex(0)[2], 0);
    _numtest(tce.vertex(1)[0], 3.0);
    _numtest(tce.vertex(1)[1], -1.5);
    _numtest(tce.vertex(1)[2], 0);
    _numtest(tce.vertex(2)[0], -1.5);
    _numtest(tce.vertex(2)[1], 3.0);
    _numtest(tce.vertex(2)[2], 0);
  }

  void DGTriangleP1Test::testShapeFct(linDG2D::TriangleP1& tel) {
    // test shape functions
    _numtest(tel.shapeFct(0, 0, 0), 1);
    _numtest(tel.shapeFct(0, 1, 0),-1);
    _numtest(tel.shapeFct(0, 1, 1),-1);
    _numtest(tel.shapeFct(1, 0, 0),-1);
    _numtest(tel.shapeFct(1, 1, 0), 1);
    _numtest(tel.shapeFct(1, 1, 1),-1);
    _numtest(tel.shapeFct(2, 0, 0),-1);
    _numtest(tel.shapeFct(2, 1, 0),-1);
    _numtest(tel.shapeFct(2, 1, 1), 1);
  }

  void DGTriangleP1Test::testShapeFctGradient(linDG2D::TriangleP1& tel) {
    // test shape function gradients
    Real dx = -4.5;
    Real dy = -4.5;
    _numtest(tel.shapeFctGradient(0)[0], 2*dx/(dx*dx+dy*dy));
    _numtest(tel.shapeFctGradient(0)[1], 2*dy/(dx*dx+dy*dy));
    dx =  5.4;
    dy = -2.7;
    _numtest(tel.shapeFctGradient(1)[0], 2*dx/(dx*dx+dy*dy));
    _numtest(tel.shapeFctGradient(1)[1], 2*dy/(dx*dx+dy*dy));
    dx = -(1.+0.2*5./26.);
    dy = 5.+.2*25./26.;
    _numtest(tel.shapeFctGradient(2)[0], 2*dx/(dx*dx+dy*dy));
    _numtest(tel.shapeFctGradient(2)[1], 2*dy/(dx*dx+dy*dy));
  }

}
