// testing linDG2D::SpaceP1

#include "toolbox.hh"
#include "basics/testsuite.hh"
#include "basics/exceptions.hh"
#include "geometry/meshImport2D.hh"
#include "spaceTest.hh"
#include "linDG2D/space.hh"
#include "linDG2D/elementPair.hh"
#include "linDG2D/edgeInfo.hh"
#include "linDG2D/meshInfo.hh"
#include "linDG2D/elementPairList.hh"

namespace test {

  using concepts::Real;
  
  // ******************************************************* DGSpace2dP1Test **

  void DGSpace2dP1Test::run() {
    try {
      concepts::ParsedFormula boundaryFrm("(0)");
      concepts::Import2dMesh mesh (SOURCEDIR "/linDG2D/testsuite/0101.coord",
				   SOURCEDIR "/linDG2D/testsuite/0101.elms",
				   SOURCEDIR "/linDG2D/testsuite/0101.bnd", 1);
      linDG2D::SpaceP1 s(mesh);

      testVertices(s, mesh);
    }
    catch (concepts::ExceptionBase& e) {
      std::cout << e << std::endl;
      _fail("caught exception");
    }
  }

  void DGSpace2dP1Test::testVertices(const linDG2D::SpaceP1& s,
                                     concepts::Mesh2& mesh) {
    linDG2D::MeshInfo mi(mesh);
    const linDG2D::ElementPairList epl(s, s, mi);
    std::auto_ptr<concepts::ElementPairList<Real>::Scanner> sc(epl.scan());
    while (*sc) {
      concepts::ElementPair<Real>& ep = (*sc)++;
      const linDG2D::ElementPair& dep =
        dynamic_cast<const linDG2D::ElementPair&>(ep);
      const linDG2D::EdgeInfo& ei = dep.edgeInfo();

      if(ei.cell(1)) {
        
        uint i0 = ei.idxs_[0];
        uint i1 = ei.idxs_[1];
        const concepts::Triangle2d& c0 = *(ei.cell(0));
        const concepts::Triangle2d& c1 = *(ei.cell(1));
        
        concepts::Vertex* ve0 = c0.connector().vertex(i0);
        concepts::Vertex* ve1 = c1.connector().vertex((i1+1)%3);
        _test(ve0 == ve1);
        
        ve0 = c0.connector().vertex((i0+1)%3);
        ve1 = c1.connector().vertex(i1);
        _test(ve0 == ve1);
        
        concepts::Real3d pe0 = c0.vertex(i0);
        concepts::Real3d pe1 = c1.vertex((i1+1)%3);
        _numtest((pe0-pe1).l2(), 0);
        
        pe0 = c0.vertex((i0+1)%3);
        pe1 = c1.vertex(i1);
        _numtest((pe0-pe1).l2(), 0);
      }
    }
  }
  
}
