// elements for linear FEM

#ifndef elementLin_hh
#define elementLin_hh

#include <memory>
#include "basics/typedefs.hh"
#include "space/element.hh"
#include "space/tmatrix.hh"
#include "toolbox/scannerConnectors.hh"

namespace concepts {
  // forward declarations
  class Connector;
  class Cell;
}

namespace linearFEM {

  using concepts::Real;

  // *************************************************************** Element **

  /** Base class for the element for linear FEM.
      @author Philipp Frauenfelder, 2002
  */
  class Element : public concepts::Element<Real> {
  public:
    /** Constructor. Creates a T matrix of type TIndex from \c idx.
	@param i Size of \c idx
	@param idx Describes the map local / global dof
    */
    Element(const uint i, uint idx[]) :
      T_(new concepts::TIndex<Real>(i,i,idx)) {}
    /** Constructor. Creates a T matrix out of \c T0 and \c T1
	@param T0 Contains T columns
	@param T1 Contains T columns
    */
    Element(concepts::TColumn<Real>* T0, concepts::TColumn<Real>* T1);
    /// Returns the support of this element
    virtual const concepts::Connector& support() const = 0;
    /// Returns the cell of this element
    virtual const concepts::Cell& cell() const = 0;
    /// Returns the T matrix of this element
    virtual const concepts::TMatrixBase<Real>& T() const { return *T_; }
  protected:
    /// T matrix of this element
    std::auto_ptr<concepts::TMatrixBase<Real> > T_;
  };

} // namespace linearFEM

namespace concepts {

  // ************************************************************** Scanners **

  template<>
  class Scan<linearFEM::Element> :
    public concepts::Scan<concepts::Element<Real> > {
  public:
    virtual linearFEM::Element& operator++(int) = 0;
  };
}

#endif // elementLin_hh
