// graphics for lines

#include <sstream>
#include "lineGraphics.hh"
#include "element1D.hh"
#include "space/postProcess.hh"
#include "graphics/meshGnuplot.hh"
#include "graphics/dataGnuplot.hh"

namespace linearFEM {

  // ********************************************************** LineGraphics **

  void LineGraphics::operator()(const concepts::Element<Real>& elm,
				enum graphicsType type,
				concepts::CellPostprocess<Real>& post) const {
    const Line* e = dynamic_cast<const Line*>(&elm);
    conceptsAssert(e, concepts::Assertion());
    switch(type) {

    case TOPOLOGY_DOTTY:
    case VERTEX_LIST:
    case MESH_MATLAB:
      post(e->cell()); break;

    case MESH_GNUPLOT: {
      graphics::MeshGnuplotCell<Real>* gnuplot =
	dynamic_cast<graphics::MeshGnuplotCell<Real>*>(&post);
      conceptsAssert(gnuplot, concepts::Assertion());

      std::ofstream* ofs = gnuplot->write();
      for (uint i = 0; i < 2; ++i)
	*ofs << e->cell().vertex(i)[0] << " " << 1 << std::endl;
      *ofs << std::endl;
      break;
    }

    case DATA_GNUPLOT: {
      graphics::DataGnuplotCell<Real>* gnuplot =
	dynamic_cast<graphics::DataGnuplotCell<Real>*>(&post);
      conceptsAssert(gnuplot, concepts::Assertion());

      const concepts::Array<Real>& coeff = gnuplot->coeff(); // coefficients

      std::ofstream* ofs = gnuplot->write();
      for (uint i = 0; i < 2; ++i) {
	*ofs << e->cell().vertex(i)[0] << " "
	     << coeff[i] << std::endl;
      }
      *ofs << std::endl;
      break;
    }

    default:
      std::stringstream error;
      error << "graphics type " << type << " not supported" << std::ends;
      throw conceptsException(concepts::MissingFeature(error.str())); break;
    }
  }

  std::ostream& LineGraphics::info(std::ostream& os) const {
    return os << "LineGraphics()";
  }

} // namespace linearFEM
