// linear form for 1D linear FEM

#include "linearForm1D.hh"
#include "element1D.hh"

namespace linearFEM {

  // *************************************************************** Riesz1d **

  void Riesz1d::operator()(const concepts::Element<Real>& elm,
			   concepts::ElementMatrix<Real>& em) {
    const uint n = elm.T().m();
    em.resize(n, 1);
    em.zeros();

    const linearFEM::Line* e = dynamic_cast<const linearFEM::Line*>(&elm);
    conceptsAssert(e, concepts::Assertion());
    const Real h = e->cell().size();

    const Real* w = quad_.weights();   // quadrature weights
    const Real* q = quad_.abscissas(); // quadrature points

    for (uint i = 0; i < n; ++i) { // loop over shape functions
      Real& m = em(i,0);
      for (uint k = 0; k < quad_.n(); ++k) { // loop over quadrature points
	m += w[k] * h / 2.0
	  * e->shapefct(i, q[k])
	  * pow(e->cell().chi( (1.0+q[k])/2.0 ), 2.0);
      } // for k
    } // for i
  }

} // namespace linearFEM
