// space for linear FEM in 3D

#include "space3D.hh"
#include "basics/debug.hh"
#include "geometry/mesh.hh"
#include "geometry/boundaryConditions.hh"
#include "toolbox/boundary.hh"

// debugging

namespace linearFEM {

  // ************************************************************** Linear3d **

  Linear3d::Linear3d(concepts::Mesh3& msh, concepts::BoundaryConditions* bc) :
    dim_(0), nelm_(0), bc_(bc), elm_(0) {

    concepts::DynArray<uint> bcMap(2, 3, 1);
    std::auto_ptr<concepts::Scan3> sc(0);
    if (bc_) {
      sc.reset(msh.scan());
      while (*sc) {
	concepts::Cell3& cell = (*sc)++;
	concepts::Connector2* face;
	for (uint j = 0; (face = cell.connector().face(j)) != 0; ++j) {
	  concepts::Connector0* vtx;
	  for (uint k = 0; (vtx = face->vertex(k)) != 0; ++k) {
	    if ((*bc_)(face->attrib()).type() == concepts::Boundary::DIRICHLET)
	      bcMap[vtx->key()] = 0;
	    else
	      bcMap[vtx->key()];
	  }
	}
      }
    }
    concepts::DynArray<uint> dofMap(2, 3, (uint)-1);
    sc.reset(msh.scan());
    while (*sc) {
      concepts::Cell3& cell = (*sc)++;
      const concepts::Tetrahedron3d* cellT =
	dynamic_cast<const concepts::Tetrahedron3d*>(&cell);
      conceptsAssert(cellT, concepts::Assertion());
      concepts::TColumn<Real>* T = 0;
      const uint n = 4;
      concepts::Connector0* vtx;
      for (uint j = 0; (vtx = cell.connector().vertex(j)) != 0; ++j) {
	if (bcMap[vtx->key()]) {
	  uint& i = dofMap[vtx->key()];
	  if (i == (uint)-1)
	    i = dim_++;
	  T = new concepts::TColumn<Real>(n, i, T);
	  (*T)[j] = 1.0;
	}
      }
      nelm_++;
      elm_ = new concepts::Joiner<Tetrahedron*, 1>
	(new Tetrahedron(*cellT, T), elm_);
    }
  }

  Linear3d::~Linear3d() {
    concepts::Joiner<Tetrahedron*, 1>::destructor(elm_);
  }

  std::ostream& Linear3d::info(std::ostream& os) const {
    return os << "Linear3d(dim = " << dim_ << ", nelm = " << nelm_ << ')';
  }


} // namespace linearFEM
