// fabric class for CG

#ifndef CGfabric_hh
#define CGfabric_hh

#if __GNUC__ == 2
#  include <float.h>
#  define EPS DBL_EPSILON
#else
#  include <limits>
#  define EPS std::numeric_limits<double>::epsilon()
#endif
#include "cg.hh"
#include "solverFabric.hh"

namespace concepts {

  // ************************************************************** CGFabric **

  /** Fabric class for conjugate gradients: \c CG.
      @author Philipp Frauenfelder, 2002
  */
  template<typename F>
  class CGFabric : public SolverFabric<F> {
  public:
    /** Constructor.
	@param maxeps Maximal residual
	@param maxit Maximal number of iterations
        @param relres Relative residual
    */	
    CGFabric(Real maxeps = EPS,
	     int maxit = 0, bool relres = false)
      : maxeps_(maxeps), maxit_(maxit), relres_(relres) {}
    virtual Operator<F>* operator()(Operator<F>& matrix) {
      return new CG<F>(matrix, maxeps_, maxit_, relres_);
    }
  protected:
    virtual std::ostream& info(std::ostream& os) const {
      return os << "CGFabric(maxeps = " << maxeps_ << ", maxit = "
		<< maxit_ << ", relres = " << relres_ << ')';
    }
  private:
    /// Convergence criteria for the solver
    Real maxeps_;
    /// Convergence criteria for the solver
    int maxit_;
    /// Convergence criteria for the solver
    bool relres_;
  };

} // concepts

#endif // CGfabric_hh
