#include "nrlSolver.hh"
#include "operator/sparseMatrix.hh"
#include "nonzeros.h"
#include "cholesky.h"

// debugging
#include "basics/debug.hh"

#define NRLSolverConstr_D 1

namespace concepts {

  NRLSolver::NRLSolver(const SparseMatrix<Real>& A) :
    n_(A.spaceX().dim()), spcX_(A.spaceY()), spcY_(A.spaceX()) {

    const HashedSparseMatrix<Real>& m = *(A.m());

    NonZeros nz = new _NonZeros;
    nz->nodes = n_;
    DEBUGL(NRLSolverConstr_D, "nodes = " << nz->nodes);
    nz->index = new uint[nz->nodes+1];

    uint adjsize = (m.used()-n_);
    DEBUGL(NRLSolverConstr_D, "adjsize = " << adjsize);
    nz->adjcnt = new uint[adjsize+10];
    nz->adjlen = adjsize;
    for (uint j = 0; j < adjsize+10; ++j) nz->adjcnt[j] = 1;

    uint hashBits = m.HashBits();
    uint p = 1<<hashBits;
    HashedSparseMatrix<Real>::Value** matrix = m.Matrix();  
    uint adjcnt = 0;
    for(uint r = 0; r < n_; ++r) {
      DEBUGL(NRLSolverConstr_D, "r = " << r);
      DEBUGL(NRLSolverConstr_D, "nz->index[" << r << "] = " << adjcnt+1);
      nz->index[r] = adjcnt+1;
      for (uint c_mod = 0; c_mod < p; ++c_mod) {
	for (HashedSparseMatrix<Real>::Value* v =
	       matrix[(r<<hashBits) + c_mod]; 
	     v != NULL; v = v->lnk){
	  if (v->idx != r) {
	    DEBUGL(NRLSolverConstr_D, "Hit a(" << r << ", " << v->idx
		   << ") = " << v->val);
	    DEBUGL(NRLSolverConstr_D, "nz->adjcnt[" << adjcnt << "] = "
		   << v->idx + 1);
	    nz->adjcnt[adjcnt++] = v->idx + 1;
	  }
	}
      }
    }
    DEBUGL(NRLSolverConstr_D, "nz->index[" << n_ << "] = " << adjcnt+1);
    nz->index[n_] = adjcnt+1;

//     const char prefix[3] = "ab";
    GlobalMatrix cofmat = CreateGM(nz, 0);
    DeleteGM(cofmat);
  }

  NRLSolver::~NRLSolver() {}

  void NRLSolver::operator()(const Function<Real>& fncY,
			     Function<Real>& fncX) {
    const Vector<Real>* fncYA;
    if ((fncYA = dynamic_cast<const Vector<Real>*>(&fncY))) {
      Vector<Real>* fncXA;
      if ((fncXA = dynamic_cast<Vector<Real>*>(&fncX))) {
	(*this)(*fncYA, *fncXA); return;
      }
    }
    throw conceptsException(MissingFeature("unknown function"));
  }

  void NRLSolver::operator()(const Vector<Real>& fncY, Vector<Real>& fncX) {
    fncX = fncY;
  }

  std::ostream& NRLSolver::info(std::ostream& os) const {
    return os << "NRLSolver(n = " << n_ << ")";
  }

} // namespace concepts
