#ifndef nrlSolver_hh
#define nrlSolver_hh

#include "basics/typedefs.hh"
#include "compositions.hh"

namespace concepts {

  // forwards
  template<typename F>
  class SparseMatrix;

  class NRLSolver : public Operator<Real> {
  public:
    NRLSolver(const SparseMatrix<Real>& A);
    ~NRLSolver();
    virtual void operator()(const Function<Real>& fncY, Function<Real>& fncX);
    void operator()(const Vector<Real>& fncY, Vector<Real>& fncX);
    virtual const Space<Real>& spaceX() const { return spcX_; }
    virtual const Space<Real>& spaceY() const { return spcY_; }
  protected:
    virtual std::ostream& info(std::ostream& os) const;
  private:
    /// System size
    const uint n_;
    /// Image space
    const Space<Real>& spcX_;
    /// Source space
    const Space<Real>& spcY_;
  };

} // namespace concepts

#endif // nrlSolver_hh
