/* testsuite for conjugate gradients
 */

#include "basics/exceptions.hh"
#include "operator/matrix.hh"
#include "operator/denseMatrix.hh"
#include "operator/sparseMatrix.hh"
#include "operator/PETSc.hh"
#include "operator/cg.hh"
#include "operator/diagonal.hh"
#include "cgTest.hh"

namespace test {

  void CgTest::run() {
    _test(spc.dim() == 5);

    concepts::DenseMatrix<Real> m(spc, spc);
    m(0,0) = 10.0; m(0,1) = m(1,0) = 0.1; m(0,3) = m(3,0) = 3.6;
    m(1,1) = 2.0;
    m(2,2) = 3.0; m(2,4) = m(4,2) = 0.9;
    m(3,3) = 4.0; m(3,4) = m(4,3) = 1.8;
    m(4,4) = 5.0;
    concepts::Vector<Real> rhs(spc);
    rhs[0] = 1.0; rhs[1] = 2.0; rhs[2] = 3.0; rhs[3] = 4.0; rhs[4] = 5.0;
    Real res[5] = { -0.28748448975485757506, 1.0143742244877425041,
		    0.85962180153593326093, 1.0481687430832675734,
		    0.46792732821355476069 };

    testNormal(m, rhs, res);
    testPrecond(m, rhs, res);

    concepts::SparseMatrix<Real> mS(spc, spc);
    mS(0,0) = 2.0; mS(0,1) = mS(1,0) = 0.3; mS(0,3) = mS(3,0) = 0.5;
    mS(1,1) = 3.0;
    mS(2,2) = 7.0; mS(2,4) = mS(4,2) = 2.0;
    mS(3,3) = 8.0; mS(3,4) = mS(4,3) = 6.0;
    mS(4,4) = 7.0;
    rhs[0] = 5.14; rhs[1] = 9.9; rhs[2] = 10.3; rhs[3] = 3.8; rhs[4] = 0.7;

    Real res2[5] = { 1.5661753759118657303, 3.1433824624088138044,
		     2.1389161828197105208, 2.1292690189072480678,
		     -2.3362066398689900204 };
    testNormal(mS, rhs, res2);
    testPrecond(mS, rhs, res2);
  }

  void CgTest::testNormal(concepts::Matrix<Real>& m,
			 concepts::Vector<Real>& rhs, Real res[5]) {
    concepts::CG<Real> cgm(m, 1e-5, 10);
    concepts::Vector<Real> sol(spc);
    cgm(rhs, sol);
    for (uint i = 0; i < 5; ++i) _numtest(sol[i], res[i]);
    cgm(rhs, sol);
    _test(cgm.iterations() == 0);
    for (uint i = 0; i < 5; ++i) _numtest(sol[i], res[i]);
  }

  void CgTest::testPrecond(concepts::Matrix<Real>& m,
				concepts::Vector<Real>& rhs, Real res[5]) {
    concepts::DiagonalMatrix<Real> diag(m);
    concepts::DiagonalSolver<Real> diagSolver(diag);
    concepts::CG<Real> Pcgm(m, diagSolver, 1e-5, 10);
    concepts::Vector<Real> sol(spc);
    Pcgm(rhs, sol);
    for (uint i = 0; i < 5; ++i) _numtest(sol[i], res[i]);
    Pcgm(rhs, sol);
    _test(Pcgm.iterations() == 0);
    for (uint i = 0; i < 5; ++i) _numtest(sol[i], res[i]);
  }

} // namespace test

int main(int, char**) {
  try {
    test::CgTest t;
    t.run();
    return t.report();
  }
  catch (concepts::ExceptionBase& e) {
    std::cout << e << std::endl;
    return 1;
  }
}
