/* testsuite for PARDISO
 */

#include "basics/exceptions.hh"
#include "operator/sparseMatrix.hh"
#include "operator/pardiso.hh"
#include "pardisoTest.hh"

using namespace concepts;

namespace test {

  void PardisoTest::run() {
    _test(spc.dim() == 5);
    testSPD(); testUNSYMM();
  }

  void PardisoTest::testSPD() {
    SparseMatrix<Real> mS(spc, spc);
    mS(0,0) = 2.0; mS(0,1) = mS(1,0) = 0.3; mS(0,3) = mS(3,0) = 0.5;
    mS(1,1) = 3.0;
    mS(2,2) = 7.0; mS(2,4) = mS(4,2) = 2.0;
    mS(3,3) = 8.0; mS(3,4) = mS(4,3) = 6.0;
    mS(4,4) = 7.0;

    Pardiso solver(mS, Pardiso::SPD);

    Vector<Real> rhs(spc), sol(spc);
    rhs[0] = 5.14; rhs[1] = 9.9; rhs[2] = 10.3; rhs[3] = 3.8; rhs[4] = 0.7;

    solver(rhs, sol);
    Real exact[5] = { 1.5661753759118659524, 3.1433824624088142485,
		      2.1389161828197100768, 2.1292690189072480678,
		      -2.3362066398689864677 };
    for (uint i = 0; i < 5; i++)
      _numtest(exact[i], sol[i]);
    sol = 0;
    solver(rhs, sol);
    for (uint i = 0; i < 5; i++)
      _numtest(exact[i], sol[i]);
  }

  void PardisoTest::testUNSYMM() {
    SparseMatrix<Real> mU(spc, spc); // same as example in Umfpack Guide p. 10
    mU(0,0) = 2; mU(0,1) = 3;
    mU(1,0) = 3; mU(1,2) = 4; mU(1,4) = 6;
    mU(2,1) = -1; mU(2,2) = -3; mU(2,3) = 2;
    mU(3,2) = 1;
    mU(4,1) = 4; mU(4,2) = 2; mU(4,4) = 1;
    Pardiso solverU(mU, Pardiso::UNSYMM);

    Vector<Real> b(spc), sol(spc);
    b[0] = 8; b[1] = 45; b[2] = -3; b[3] = 3; b[4] = 19;
    solverU(b, sol);
    Real exact[5] = { 1, 2, 3, 4, 5 };
    for (uint i = 0; i < 5; i++)
      _numtest(exact[i], sol[i]);
    sol = 0;
    solverU(b, sol);
    for (uint i = 0; i < 5; i++)
      _numtest(exact[i], sol[i]);
  }

} // namespace test

int main(int, char**) {
  try {
    test::PardisoTest t;
    t.run();
    return t.report();
  }
  catch (ExceptionBase& e) {
    std::cout << e << std::endl;
    return 1;
  }
}
