/* Elements
 * Elements to build a space
 */

#include "element.hh"
#include "basics/debug.hh"
#include <string.h>
#include <iomanip>
#ifdef __SUNPRO_CC
#  include "toolbox/scannerConnectors.hh"
#endif

// debugging ElementMatrix
#define ElementMatrixConstr_D 0
#define ElementMatrixDestr_D 0
#define ElementMatrixResize_D 0

namespace concepts {

#ifdef __SUNPRO_CC
  template class PListScan<Element<Real> >;
#endif

  // ********************************************************* ElementMatrix **

  template<class F>
  ElementMatrix<F>& ElementMatrix<F>::operator=(const ElementMatrix<F>& A) {
    if (&A != this) {
      resize(A.m_, A.n_);
      std::memcpy((F*)data_, (const F*)A.data_, n_*m_*sizeof(F));
      t_ = A.t_;
    }
    return *this;
  }

  template<class F>
  void ElementMatrix<F>::resize(uint m, uint n) {
    DEBUGL(ElementMatrixResize_D, "old: m = " << m_ << ", n = " << n_);
    data_.resize(n*m);
    n_ = n;
    m_ = m;
    t_ = false;
    DEBUGL(ElementMatrixResize_D, "new: m = " << m_ << ", n = " << n_);
  }

  template<class F>
  std::ostream& ElementMatrix<F>::info(std::ostream& os) const {
    const int N = n();
    const int M = m();
    os << "ElementMatrix(" << M << 'x' << N << ", [" << std::endl;
    for (int i = 0; i < M; ++i) {
      for (int j = 0; j < N; ++j) {
  	os << std::setw(2) << operator()(i,j) << ((j == N-1) ? "" : " ");
      } // for j
      os << ((i == M-1) ? "" : "\n");
    } // for i
    return os << "])";
  }

  // **************************************************** BlockElementMatrix **

  template<class F>
  std::ostream& BlockElementMatrix<F>::info(std::ostream& os) const {
     const int N = em_.n();
     const int M = em_.m();
     os << "BlockElementMatrix(" << M << 'x' << N <<
       ", active block = (" << offm_ << ".." << offm_+bm_-1 << "," 
	<<  offn_ << ".." << offn_+bn_-1 << ") , [" << std::endl;
     for (int i = 0; i < M; ++i) {
       for (int j = 0; j < N; ++j) {
   	os << std::setw(2) << em_.operator()(i,j) << ((j == N-1) ? "" : " ");
       } // for j
       os << ((i == M-1) ? "" : "\n");
     } // for i
     return os << "])";
  }


} // namespace concepts
