/* include files for sparseqr package
 */

/*----------------------------------------------------------------------------
Sparse QR solver for linear systems of equations or least squares problems
Copyright (C) 1996, 1997, 1998 Thomas H. Robey

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Inquiries can be directed to trobey@arc.unm.edu or

Thomas H. Robey
925 Madison NE
Albuquerque, NM  87110
USA
----------------------------------------------------------------------------*/

#ifndef sparseqr_inc_hh
#define sparseqr_inc_hh

#include "sparseqr/sparseqr.hh"
#include "sparseqr/givensRotations.hh"
#include "sparseqr/driver.hh"

/** SparseQR solver.

    From the README file:<br>
    This routine computes the QR decomposition of a mxn matrix
    A, m>=n, such that
    \f[ P_r \cdot Q^\top \cdot A \cdot P_c^\top =
        \begin{pmatrix} R \\ 0 \end{pmatrix} \f]
    where \f$Q^\top \cdot Q = I \in R^{m\times m}\f$, 
    \f$R \in R^{m\times m}\f$ is an upper triangular mxm matrix and
    \f$P_r\in R^{m\times m}\f$ as well as \f$P_c\in R^{n\times n}\f$
    are permutation matrices.

    @author Thomas H. Robey, 1996, 1997, 1998
    @see <a href="http://www.ahpcc.unm.edu/~trobey/#SPARSEQR">Homepage of SparseQR</a><br>
    Thomas H. Robey and Deborah L. Sulsky, <i>Row Ordering for Sparse QR Decomposition,</i> SIAM J. Matrix Anal. and Appl., Vol. 15, No. 4, pp. 1208-1225, October 1994.
*/
namespace sparseqr {
  // just for documentation, leave empty
}

#endif // sparseqr_inc_hh
