/*----------------------------------------------------------------------------
Sparse QR solver for linear systems of equations or least squares problems
Copyright (C) 1996, Thomas H. Robey

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Inquiries can be directed to trobey@arc.unm.edu or

Thomas H. Robey
925 Madison NE
Albuquerque, NM  87110
USA
----------------------------------------------------------------------------*/

#ifndef mindegree_hh
#define mindegree_hh

#include <stddef.h>

namespace sparseqr {

  /** @author Thomas H. Robey, 1996
      @see <a href="http://www.ahpcc.unm.edu/~trobey/#SPARSEQR">Homepage of SparseQR</a><br>
      Thomas H. Robey and Deborah L. Sulsky, <i>Row Ordering for Sparse QR Decomposition,</i> SIAM J. Matrix Anal. and Appl., Vol. 15, No. 4, pp. 1208-1225, October 1994.
  */
  class MinDegree {
  public:
    MinDegree(QR *,int);
    ~MinDegree() {
      delete[] degree;
    };
    int MinSearch(QR *,int,int,int *);
    void Track(QR *,int,int,int);
    void Untrack(QR *,int,int,int,int);
  private:
    int *degree,*list;
    int *work;
  };

} // namespace sparseqr

#endif // mindegree_hh
