/*----------------------------------------------------------------------------
Sparse QR solver for linear systems of equations or least squares problems
Copyright (C) 1996, Thomas H. Robey

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Inquiries can be directed to trobey@arc.unm.edu or

Thomas H. Robey
925 Madison NE
Albuquerque, NM  87110
USA
----------------------------------------------------------------------------*/

#include <cassert>
#include "sparseqr.hh"

namespace sparseqr {

  Packed::Packed(unsigned cnt) {
    size=cnt;
    if (cnt!=0) {
      col=new int[cnt];
      value=new double[cnt];
      assert(col!=NULL && value!=NULL);
    }
  };

  void Packed::init(unsigned cnt) {
    size=cnt;
    if (cnt!=0) {
      col=new int[cnt];
      value=new double[cnt];
      assert(col!=NULL && value!=NULL);
    }
  };

  Packed::~Packed() {
    if (size!=0) {
      size=0;
      delete[] col;
      delete[] value;
    }
  };

  void Packed::Sort() {
    int k=0,sup,employee;
    double t=0.0;
    if (size<=1)
      return;
    int level=1;
    while (1<<level<=(int) size)
      level++;
    int end=size;
    while (end>1) {
      if (level>1)
	level--;
      /* Promotion phase */
      else {
	end--;
	k=col[end];
	col[end]=col[0];
	t=value[end];
	value[end]=value[0];
      }
      for (int i=1<<(level-1);(i<1<<level && i<<1<=end);i++) {
	sup=i;
	/* Hiring phase */
	if (end==(int) size) {
	  k=col[sup-1];
	  t=value[sup-1];
	}
	while (sup<<1<=end) {
	  employee=sup<<1;
	  /* Find best employee */
	  if (employee<end && col[employee]>col[employee-1])
	    employee++;
	  /* Promote employee */
	  if (col[employee-1]>k) {
	    col[sup-1]=col[employee-1];
	    value[sup-1]=value[employee-1];
	    sup=employee;
	  }
	  else {
	    employee=sup;
	    break;
	  }
	}
	/* Place person at their level */
	if (level==1 || sup!=i) {
	  col[employee-1]=k;
	  value[employee-1]=t;
	}
      }
    }
    col[0]=k;
    value[0]=t;
    return;
  };

  std::ostream& operator<<(std::ostream& os, const Packed& p) {
    os << "Packed(" << p.size;
    if (p.size > 0) {
      os << ", ";
      for (unsigned i = 0; i < p.size; ++i) {
	os << '(' << p.col[i] << ", " << p.value[i] << ')';
	if (i < p.size-1) os << ", ";
      }
    }
    return os << ')';
  }

}
