/*----------------------------------------------------------------------------
Sparse QR solver for linear systems of equations or least squares problems
Copyright (C) 1996, Thomas H. Robey

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Inquiries can be directed to trobey@arc.unm.edu or

Thomas H. Robey
925 Madison NE
Albuquerque, NM  87110
USA
----------------------------------------------------------------------------*/

#include "sparseqr.hh"

namespace sparseqr {

  Qmatrix::Qmatrix() {
    qbegin=NULL;
    qend=NULL;
  };

  Qmatrix::~Qmatrix() {
    J *jpptr;
    J *jptr=qbegin;
    while (jptr!=NULL) {
      jpptr=jptr;
      jptr=jptr->next;
      delete jpptr;
    }
    qbegin=NULL;
    qend=NULL;
  };

  double *Qmatrix::ApplyQ(double *b) {
    double t;
    J *jptr=qbegin;
    while (jptr!=NULL) {
      t=b[jptr->row1];
      b[jptr->row1]=jptr->c*t+jptr->s*b[jptr->row2];
      b[jptr->row2]=jptr->c*b[jptr->row2]-jptr->s*t;
      jptr=jptr->next;
    }
    return(b);
  };

  std::ostream& operator<<(std::ostream& os, const Qmatrix& q) {
    os << "Qmatrix(";
    J* jptr = q.qbegin;
    while (jptr != 0) {
      os << *jptr;
      jptr=jptr->next;
      if (jptr != 0) os << ", ";
    }
    return os << ')';
  }

}
