/*----------------------------------------------------------------------------
Sparse QR solver for linear systems of equations or least squares problems
Copyright (C) 1996, Thomas H. Robey

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Inquiries can be directed to trobey@arc.unm.edu or

Thomas H. Robey
925 Madison NE
Albuquerque, NM  87110
USA
----------------------------------------------------------------------------*/

/******************************************************************************
*  Lexigraphical input analyzer
*  Purpose:  Allows commented input files
*  Author: Tom Robey, Spectra Research, 5-6-91
*******************************************************************************/
#include <iostream>
#include <fstream>
#include <cctype>
#include <cstdlib>
#include "lex.hh"

using namespace std;

int lex(void);
char lex_text[81];
ifstream lexFile;
/***************************************************************************
*  Procedure:  lex
*  Function:  Return the next lexical token in the current input stream
*  Input:  None
*  Return:  Length of the parsed token (0 indicates error)
*  Exit conditions:  lex_text contains the (zero terminated) text of the
*    token
***************************************************************************/
int lex()
{
  char c;
  int i,j;
  do {
    j=lexFile.get();
    if (lexFile.eof() || (j==-1))
      return(0);
    c=(char) j;
    if (c=='#') {
      j=lexFile.get();
      if (lexFile.eof() || (j==-1))
	return(0);
      c=(char) j;
      if (c=='#')
	break;
      while (c!='#') {
	j=lexFile.get();
	if (lexFile.eof() || (j==-1))
	  return(0);
        c=(char) j;
      }
      j=lexFile.get();
      if (lexFile.eof() || (j==-1))
	return(0);
      c=(char) j;
    }
  } while (c==' ' || c=='\t' || c=='\n');
  i=0;
  while (c!=' ' && c!='\t' && c!='\n'  && c!='\r' && i<81) {
    lex_text[i]=c;
    i++;
    j=lexFile.get();
    if (lexFile.eof() || (j==-1)) {
      lex_text[i]='\0';
      return(i);
    }
    c=(char) j;
  }
  lex_text[i]='\0';
  return(i);
}
/* Read integer input */
int read_integer(int *int_variable)
{
  int i,j;
  j=lex();
  for (i=0;i<j;i++) {
    if (isdigit(lex_text[i])==0  && lex_text[i]!='-')
      j=0;
  }
  if (j==0) {
    cerr << "Syntax error, integer number expected\n";
    return(-1);
  }
  else {
    *int_variable=atoi(lex_text);
    return(0);
  }
}
/* Read unsigned input */
int read_unsigned(unsigned *unsigned_variable)
{
  int i,j;
  j=lex();
  for (i=0;i<j;i++) {
    if (isdigit(lex_text[i])==0)
      j=0;
  }
  if (j==0) {
    return(-1);
  }
  else {
    j=atoi(lex_text);
    if (j<0)
      cerr << "Syntax error, unsigned integer expected\n";
    else  
      *unsigned_variable=(unsigned) j;
    return(0);
  }
}
/* Read double precision input */
int read_double(double *double_variable)
{
  int i,j;
  j=lex();
  i=0;
  if (i<j && (lex_text[i]=='-' || lex_text[i]=='+'))
    i++;
  while (i<j && isdigit(lex_text[i]))
    i++;
  if (i<j && lex_text[i]=='.')
    i++;
  while (i<j && isdigit(lex_text[i]))
    i++;
  if (i<j && (lex_text[i]=='e' || lex_text[i]=='E'))
    i++;
  if (i<j && (lex_text[i]=='-' || lex_text[i]=='+'))
    i++;
  while (i<j && isdigit(lex_text[i]))
    i++;
  if (j==0 || i!=j) {
    cerr << "Syntax error, floating point number expected\n";
    return(-1);
  }
  else {
    *double_variable=atof(lex_text);
    return(0);
  }
}

