// timestepping

#include "timestepping.hh"
#include "function/vector.hh"
#include "operator/compositions.hh"
#include "strategy.hh"

// debugging
#include "basics/debug.hh"

#define stepAppl_D 0

namespace timestepping {

  // ********************************************************** Timestepping **

  TimeStepping::TimeStepping(TimeStepStrategy& timeStepStrategy) : 
    n_(0),
    timeStepStrategy_(timeStepStrategy) {}

  void TimeStepping::operator()(concepts::Vector<Real>& sol, uint n) {
    if (n < n_)
      throw conceptsException
	(concepts::MissingFeature("cannot go back in time"));
    while (n > n_) {
      timeStepStrategy_.next();
      DEBUGL(stepAppl_D, "rhs = " << timeStepStrategy_.rhs_ << std::endl);
      DEBUGL(stepAppl_D, "sol = " << timeStepStrategy_.sol_ << std::endl);
      ++n_;
    }
    sol = timeStepStrategy_.sol_;
  }

  uint TimeStepping::operator()(concepts::Vector<Real>& sol) {
    sol = timeStepStrategy_.sol_;
    return n_;
  }
  
  Real TimeStepping::time() const { return timeStepStrategy_.t_; }

  std::ostream& TimeStepping::info(std::ostream& os) const {
    return os << "TimeStepping(strategy = " << timeStepStrategy_
	      << ", n = " << n_ << ')';
  }

}
