#include <sstream>
#include "cgUzawa.hh"
#include "basics/exceptions.hh"
#include "operator/cg.hh"

// debugging

#define CGUzawaAppl_D 0

namespace vectorial {

  // *************************************************************** CGUzawa **

  void CGUzawa::operator()(const concepts::Function<Real>& fncY,
                           concepts::Function<Real>& fncX) {
    const concepts::Vector<Real>* fncYA;
    if ((fncYA = dynamic_cast<const concepts::Vector<Real>*>(&fncY))) {
      concepts::Vector<Real>* fncXA;
      if ((fncXA = dynamic_cast<concepts::Vector<Real>*>(&fncX))) {
        (*this)(*fncYA, *fncXA); return;
      }
    }
    throw conceptsException(concepts::MissingFeature
                            ("function not supported"));
  }

  void CGUzawa::operator()(const concepts::Vector<Real>& fncY,
                           concepts::Vector<Real>& fncX) {
    // subspaces
    const concepts::Space<Real>& spcX = *(spc_.get(0));
    const concepts::Space<Real>& spcY = *(spc_.get(1));

    // subvectors
    concepts::Vector<Real> x(spcX, (Real*)(fncX));
    concepts::Vector<Real> y(spcY, (Real*)(fncX)+spcX.dim());
    const concepts::Vector<Real> f(spcX, (Real*)(fncY));
    const concepts::Vector<Real> g(spcY, (Real*)(fncY)+spcX.dim());

    // reduced matrix
    concepts::Compose<Real> AiB(Ai_, B_);
    concepts::Compose<Real> BtAiB(Bt_, AiB);
    concepts::LiCo<Real> rA(BtAiB, C_);

    // reduced rhs
    concepts::Vector<Real> yRhs(g);
    concepts::Vector<Real> xRhs(f);
    Ai_(f, xRhs);
    Bt_(xRhs, yRhs);
    yRhs -= g;

    // solve for y component
    concepts::CG<Real> ySolver(rA, maxeps_, maxit_, relres_);
    ySolver(yRhs, y);
    it_ = ySolver.iterations();
    eps_ = ySolver.epsilon();

    // corrected rhs
    B_(y, xRhs);
    xRhs *= -1.;
    xRhs += f;

    // solve for x component
    Ai_(xRhs, x);
  }

  std::ostream& CGUzawa::info(std::ostream& os) const {
    return os << "CGUzawa(A = " << A_ << ", B = " << B_ << ", C = "
              << C_ << "Ai = " << Ai_ << ", eps = " << eps_
              << ", it = " << it_ << ", relres = " << relres_ << ')';
  }

} // namespace vectorial
