// graphics for vector valued elements

#ifndef vectorGraphics_hh
#define vectorGraphics_hh

#include "toolbox/elementGraphics.hh"

namespace vectorial {

  // ************************************************************** Graphics **

  /** Handles graphics for vector valued elements.
      @author Philipp Frauenfelder, 2002
  */
  template<typename F>
  class Graphics : public concepts::ElementGraphics<F> {
  public:
    Graphics() : idx_(0) {}
    /** Choose component to be plotted.
	@param idx Chooses the component \c idx of the vector valued elements
	           to be visualized
    */
    void choose(uint idx) { idx_ = idx; }
    /// Returns index of chosen component
    uint chosen() const { return idx_; }
    virtual void operator()
      (const concepts::Element<F>& elm,
       typename concepts::ElementGraphics<F>::graphicsType type,
       concepts::CellPostprocess<F>& post) const;
  protected:
    virtual std::ostream& info(std::ostream& os) const;
  private:
    /// Component which should be visualized
    uint idx_;
  };

} // namespace vectorial

#endif // vectorGraphics_hh
