// linear form for vector valued problems

#ifndef linearFormV_hh
#define linearFormV_hh

#include "vectorial.hh"
#include "function/linearForm.hh"

namespace vectorial {

  // ************************************************************ LinearForm **

  /** Vector valued linear form.
      @author Kersten Schmidt, 2002
  */
  template<class F>
  class LinearForm : public Vectorial<concepts::LinearForm<F> >,
		     public concepts::LinearForm<F> {
  public:
    LinearForm(const uint vdim, const uint arrayWidth=0) : 
      Vectorial<concepts::LinearForm<F> >(vdim,vdim) {
      DEBUGL(LinearFormConstr_D, "done.");
    }
    virtual ~LinearForm();

    /** insert a Linearform lf of one vectorial dimension to the all 
	v-dimensionial Linearform

	@param i  row number of v-dimensions
    */
    virtual void put(concepts::LinearForm<F> &lf,const int i,const int j=0);
    
    virtual void operator ()(const concepts::Element<F>& elm,
			     concepts::ElementMatrix<F>& em);
  protected:
    virtual std::ostream& info(std::ostream& os) const;
  };

} // namespace vectorial

#endif // linearFormV_hh
