// Helper class for LinearForm: assembling of components

#include "linearFormHelper.hh"

// debugging
#include "basics/debug.hh"

#define lfHelperassComp_D 0

namespace vectorial {

  // ****************************************************** LinearFormHelper **

  void LinearFormHelper::assembleComponents
  (const Element<Real>& elm, concepts::ElementMatrix<Real>& em,
   concepts::LinearForm<Real>** const lfs, const uint vdim) {
    concepts::ElementMatrix<Real> A(0,0);    // part of the em
    uint offsetColumn = 0;

    // put the size to the number of local degrees of freedom
    em.resize(elm.T().m(),1);
    DEBUGL(lfHelperassComp_D,"em: " << em.m() << "x" << em.n());

    for (uint i = 0; i < vdim; ++i) {
      const concepts::Element<Real> &e = *(elm.get(i));
      // calculate only for existing scalar linear forms
      if (lfs[i] != 0) {
        concepts::LinearForm<Real>& lf = *(lfs[i]);
        lf(e, A);
        DEBUGL(lfHelperassComp_D,"A = " << A << ", offsetColumn = "
               << offsetColumn);
        
        uint k_global = offsetColumn;
        for (uint k = 0; k < A.m(); ++k) {
          DEBUGL(lfHelperassComp_D, "k = " << k << ", k = " << k_global);
          em(k_global, 0) = A(k, 0);
          ++k_global;
        }
      }
      offsetColumn += e.T().m();
    }
  }

} // namespace vectorial
